// ------------------------------------------------------------------
//  $Id: application_copy.h 1348 2013-08-16 00:45:07Z secanell $
//
//  Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat.
//  Copyright (C) 2012 by Valentin N. Zingan, University of Alberta.
//
//  This file is subject to QPL and may not be  distributed
//  without copyright and license information. Please refer
//  to the file deal.II/doc/license.html for the  text  and
//  further information on this license.
// ------------------------------------------------------------------

#ifndef _DEALII_APPFRAME_APPLICATION_COPY_H_
#define _DEALII_APPFRAME_APPLICATION_COPY_H_

#include <appframe/application_base.h>

using namespace dealii;

namespace AppFrame
{

/**
 * Base class for non-terminal applications. Whenever a derived class
 * does not implement a function of the interface of ApplicationBase,
 * this function is forwarded to the next application in the chain.
 */

class ApplicationCopy : public ApplicationBase
{
public:

  /**
   * Constructor for a derived
   * application. A SmartPointer
   * to <tt>app</tt> is stored in
   * the copy. Therefore,
   * <tt>app</tt> must live
   * longer than the
   * ApplicationCopy object.
   */
  ApplicationCopy(ApplicationBase& ex_app);

 ~ApplicationCopy() { };

  virtual void declare_parameters(ParameterHandler& param) { app->declare_parameters(param);             }

  virtual void initialize(ParameterHandler& param)         { app->initialize(param);                     }

  virtual void remesh()                                    { app->remesh();                              }

  virtual void init_vector(FEVector& dst) const            { app->init_vector(dst);                      }

  virtual void start_vector(FEVector& dst,
                            std::string caller) const      { app->start_vector(dst, caller);             }

  virtual double residual(FEVector& dst,
                          const FEVectors& src)            { return app->residual(dst, src);             }

  virtual void solve(FEVector& start,
                     const FEVectors& rhs)                 { app->solve(start, rhs);                     }

  virtual void Tsolve(FEVector& start,
                      const FEVectors& rhs)                { app->Tsolve(start, rhs);                    }

  virtual double estimate(const FEVectors& src)            { return app->estimate(src);                  }

  virtual double evaluate(const FEVectors& src)            { return app->evaluate(src);                  }

  virtual void grid_out(const std::string& filename) const { app->grid_out(filename);                    }

  virtual void data_out(const std::string& filename,
                        const FEVectors& src)              { app->data_out(filename, src);               }

  virtual std::string id() const                           { return std::string( typeid(*this).name() ); }

  virtual void notify(const Event& reason)                 {
                                                             ApplicationBase::notify(reason);
                                                             app->notify(reason);
                                                           }

protected:

  /**
   * Pointer to the application
   * this one depends upon.
   */
  SmartPointer<ApplicationBase> app;
};

inline
ApplicationCopy::ApplicationCopy(ApplicationBase& ex_app)
:
ApplicationBase(ex_app.get_data()),
app(&ex_app)
{
  deallog << "->Copy";
}

}

#endif