//---------------------------------------------------------------------------
//    $Id: constrained_matrix.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __APPFRAME_CONSTRAINED_MATRIX_H
#define __APPFRAME_CONSTRAINED_MATRIX_H

#include <boost/shared_ptr.hpp>
#include <lac/pointer_matrix.h>
#include <lac/constraint_matrix.h>


template <class VECTOR>
class ConstrainedMatrix :
  public Subscriptor
{
  public:
    template <class MATRIX>
    void initialize(const MATRIX& matrix, ConstraintMatrix& constraints, bool reverse);
    void clear();
    
    void vmult(VECTOR& dst, const VECTOR& src) const;
    void vmult_add(VECTOR& dst, const VECTOR& src) const;
    void Tvmult(VECTOR& dst, const VECTOR& src) const;
    void Tvmult_add(VECTOR& dst, const VECTOR& src) const;

    boost::shared_ptr<PointerMatrixBase<VECTOR> > matrix;
    SmartPointer<ConstraintMatrix> constraints;
    bool reverse;
};



template <class VECTOR>
template <class MATRIX>
void
ConstrainedMatrix<VECTOR>::initialize(const MATRIX& m, ConstraintMatrix& c, bool r)
{
  matrix = boost::shared_ptr<PointerMatrixBase<VECTOR> > (new_pointer_matrix_base(m, VECTOR()));
  constraints = &c;
  reverse = r;
}



template <class VECTOR>
void
ConstrainedMatrix<VECTOR>::clear()
{
  matrix = 0;
  constraints = 0;
}



template <class VECTOR>
void
ConstrainedMatrix<VECTOR>::vmult(VECTOR& dst, const VECTOR& src) const
{
  GrowingVectorMemory<VECTOR> mem;
  VECTOR* tmp = mem.alloc();
  tmp->reinit(src, true);
  *tmp = src;
  
  if (reverse)
    constraints->condense(*tmp);
  else
    constraints->distribute(*tmp);
  
  matrix->vmult(dst, *tmp);
  
  if (reverse)
    constraints->distribute(dst);
  else
    constraints->condense(dst);
  
  mem.free(tmp);
}



template <class VECTOR>
void
ConstrainedMatrix<VECTOR>::vmult_add(VECTOR& dst, const VECTOR& src) const
{
  GrowingVectorMemory<VECTOR> mem;
  VECTOR* tmp = mem.alloc();
  VECTOR* tmp2 = mem.alloc();
  tmp->reinit(src, true);
  *tmp = src;
  
  if (reverse)
    constraints->condense(*tmp);
  else
    constraints->distribute(*tmp);
  
  matrix->vmult(*tmp2, *tmp);

  if(reverse)
    constraints->distribute(*tmp2);
  else
    constraints->condense(*tmp2);
  
  dst.add(1., *tmp2);
  mem.free(tmp2);
  mem.free(tmp);
}



template <class VECTOR>
void
ConstrainedMatrix<VECTOR>::Tvmult(VECTOR& dst, const VECTOR& src) const
{
  GrowingVectorMemory<VECTOR> mem;
  VECTOR* tmp = mem.alloc();
  tmp->reinit(src, true);
  *tmp = src;
//  constraints->distribute(*tmp);
  matrix->Tvmult(dst, *tmp);
  constraints->condense(dst);
  mem.free(tmp);
}



template <class VECTOR>
void
ConstrainedMatrix<VECTOR>::Tvmult_add(VECTOR& dst, const VECTOR& src) const
{
  GrowingVectorMemory<VECTOR> mem;
  VECTOR* tmp = mem.alloc();
  VECTOR* tmp2 = mem.alloc();
  tmp->reinit(src, true);
  *tmp = src;
//  constraints->distribute(*tmp);
  matrix->Tvmult(*tmp2, *tmp);
  constraints->condense(*tmp2);
  dst.add(1., *tmp2);
  mem.free(tmp2);
  mem.free(tmp);
}

#endif
