//---------------------------------------------------------------------------
//    $Id: doc.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2008 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

/**
 * \mainpage[deal.II application framework and shop]
 *
 * This directory currently contains two more or less separate
 * entities, one is the #AppShop and namespaces thereunder, which
 * contains a library assemble loops for local matrices and residuals
 * for different operators. These are pretty straight forward and
 * should not need too much introduction. They can be used for
 * instance in LocalIntegrator.
 *
 * <h3>The application framework</h3>
 *
 * The major part is the application framework in the namespace
 * #AppFrame. It is based on the idea that most algorithms can be
 * broken down into the evaluation of functionals, forms and the
 * solution of linear or nonlinear problems. Here, in a very abstract
 * way, a functional returns one or several numbers and a form maps
 * vectors into the dual space.
 *
 * Based on this concept, a few algorithms have been implemented yet:
 * AppFrame::MeshLoop, AppFrame::Newton, AppFrame::ThetaTimestepping
 * and AppFrame::Random.
 *
 * At some point, a finite element code has to actually implement
 * these operations. This is, where AppFrame::DoFApplication comes
 * into play. The function AppFrame::DoFApplication::integrate_1form()
 * for instance provides the loop over all mesh cells and faces and
 * calls the functions of AppFrame::LocalIntegrator to perform the
 * integration on each mesh object. It is used for instance by the
 * class AppFrame::DoFResidual.
 *
 * <h3>A note on the assembling of matrices</h3>
 *
 * This used to be part of the interface of an application, but has
 * been removed. Instead, it is left to the function
 * AppFrame::ApplicationBase::solve() of a derived class to make sure
 * that there is a matrix to be inverted if so desired. Since on the
 * other hand assembling matrices is a time-consuming task, it is
 * desirable that Newton can give advice to this function whether it
 * should reassemble or not ,depending on its convergence. To this
 * end, AppFrame::ApplicationBase::Event has been added, which can be
 * recorded using AppFrame::ApplicationBase::notify(). It is up to the
 * actual implementation to handle the events that have accumulated.
 */
