//---------------------------------------------------------------------------
//    $Id: matrix_base.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __deal2__appframe__matrix_base_h
#define __deal2__appframe__matrix_base_h

#include <appframe/base.h>
#include <lac/full_matrix.h>
#include <grid/tria.h>
#include <fe/fe_values.h>



namespace AppFrame
{
/**
 * A wrapper around a matrix object, storing the coordinates in a
 * block matrix as well.
 *
 * @note They are the position in the global matrix, where I don't use
 * BlockSparseMatrix. Matrices may have the same coordinates for the
 * following reason: A preconditioner for the Oseen system can be
 * built as a block system, where the pressure block is of the form
 * M^-1 F A^-1 with m the mass matrix (u,v), A the Laplacian and F the
 * advection diffusion operator. So, I need to build three matrices
 * for a single block in my system + in some cases a pressure
 * stabilization.

* @author Guido Kanschat, 2006
 */
  template <class MATRIX>
  struct MatrixBlock
  {
				       /**
					* Row coordinate.  This is the
					* position of the data member
					* matrix on the global matrix.
					*/
      unsigned int row;
				       /**
					* Column coordinate.  This is
					* the position of the data
					* member matrix on the global
					* matrix.
					*/
      unsigned int column;
				       /**
					* The matrix itself
					*/
      MATRIX matrix;
				       /**
					* Constructor.
					*/
    MatrixBlock()
		      :
		      row(deal_II_numbers::invalid_unsigned_int),
		      column(deal_II_numbers::invalid_unsigned_int)
    {}
    
				       /**
					* Constructor.
					*/
    MatrixBlock(const MatrixBlock<MATRIX>& M)
		      :
		      row(M.row),
		      column(M.column),
		      matrix(M.matrix)
    {}
    
				       /**
					* Constructor.
					*/
    
    MatrixBlock(unsigned int i, unsigned int j)
		      :
		      row(i), column(j)
    {}
  };
        			   /**
				    * The matrix vector handed to the
				    * local integration routines in
				    * MatrixApplication::assemble().
				    */
  typedef std::vector<MatrixBlock<FullMatrix<double> > > MatrixVector;
  
				   /**
				    * The vector of vectors handed to
				    * the local integration routines
				    * in residual().
				    */
  typedef std::vector<Vector<double> > VectorVector;
  
}


#endif
