//---------------------------------------------------------------------------
//    $Id: matrix_shop_bdry.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __appshop__matrix_shop_bdry_h
#define __appshop__matrix_shop_bdry_h

#include <appframe/app_shop.h>
#include <appframe/matrix_base.h>

/**
 * \file
 * \brief Matrix integration routines for boundary fluxes for standard bilinear forms.
 */
using namespace AppFrame;


namespace AppShop
{
  namespace Matrix
  {
    namespace Boundary
    {
				       /**
					* Upwind flux at the boundary.
					*/
      template <int dim>
      void advection(
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	const std::vector<Point<dim> >& velocity,
	double factor = 1.);
      
				       /**
					* Upwind flux at the boundary.
					*/
      template <int dim>
      void advection(
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	const VectorSlice<const std::vector<std::vector<double> > >& velocity,
	double factor = 1.);
      
				       /**
					* Scalar product of trial
					* function and normal vector on
					* the boundary.
					*/
      template <int dim>
      void u_dot_n (
	std::vector<MatrixBlock<FullMatrix<double> > >& M,
	const std::vector<unsigned int>& indices,
	const FEFaceValuesBase<dim>& fe,
	const FEFaceValuesBase<dim>& fetest,
	double factor = 1.);
      
				       /**
					* Scalar product of trial
					* function and normal vector on
					* the boundary.
					*/
      template <int dim>
      void u_dot_n_scaled (
	std::vector<MatrixBlock<FullMatrix<double> > >& M,
	const std::vector<unsigned int>& indices,
	const FEFaceValuesBase<dim>& fe,
	const FEFaceValuesBase<dim>& fetest,
	const std::vector<double>& factor,
	bool divide = false);
      
				       /**
					* u times a single component of
					* the normal vector.
					*/
      template <int dim>
      void u_ni (
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	const FEFaceValuesBase<dim>& fetest,
	unsigned int i);
      
				       /**
					* The boundary form discretizing
					*
					* \f[
					* \left<a u,v\right>
					* \f]
					*/
      template <int dim>
      void jump (
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	const double a = 1.);
				       /**
					* Boundary mass matrix.
					*/
      template <int dim>
      void jump_scaled (
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	const std::vector<double>& factor,
	const double factor2 = 1.,
	bool divide = false);
				       /**
					* Weak boundary condition of Nitsche.
					*/
      template <int dim>
      void nitsche (
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	double penalty,
	double factor = 1.);
      
				       /**
					* Weak boundary condition of Nitsche.
					*/
      template <int dim>
      void nitsche_scaled (
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	const std::vector<double>& factor,
	double penalty);
      
				       /**
					* Elasticity
					*/
      template <int dim>
      void nitsche_div (
	FullMatrix<double>& M,
	const FEFaceValuesBase<dim>& fe,
	double penalty);
    }
  }
}


#endif
