//---------------------------------------------------------------------------
//    $Id: matrix_shop_face.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __appshop__matrix_shop_face_h
#define __appshop__matrix_shop_face_h

#include <appframe/app_shop.h>
#include <appframe/matrix_base.h>

/**
 * \file
 * \brief Matrix integration routines for face fluxes for standard bilinear forms.
 */
using namespace AppFrame;

namespace AppShop
{
  namespace Matrix
  {
    namespace Face
    {
				       /**
					* Upwind flux at interior faces.
					*/
      template <int dim>
      void advection (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const VectorSlice<const std::vector<std::vector<double> > >& velocity,
	double factor = 1.);

				       /**
					* Upwind flux at interior faces.
					*/
      template <int dim>
      void advection (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const std::vector<Point<dim> >& velocity);

/**
 * Normal flux.
 *
 * \f[
 * -\left<\{u\}, [v n]\right>
 * \f]
 */
      template <int dim>
      void u_times_n (
	std::vector<MatrixBlock<FullMatrix<double> > >& intint,
	std::vector<MatrixBlock<FullMatrix<double> > >& intext,
	std::vector<MatrixBlock<FullMatrix<double> > >& extint,
	std::vector<MatrixBlock<FullMatrix<double> > >& extext,
	const std::vector<unsigned int>& indices,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const FEFaceValuesBase<dim>& int_fetest,
	const FEFaceValuesBase<dim>& ext_fetest,
	double factor = 1.);
      
				       /**
					* Flux <i>[n.u]{v}</i>.
					*/
      template <int dim>
      void u_times_n_scaled (
	FullMatrix<double>& intint,
	FullMatrix<double>& intext,
	FullMatrix<double>& extint,
	FullMatrix<double>& extext,
	const FEFaceValuesBase<dim>& int_fetest,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fetest,
	const FEFaceValuesBase<dim>& ext_fe,
	const std::vector<double>& int_factor,
	const std::vector<double>& ext_factor);

				       /**
					* Flux for superconvergent LDG.
					*/
      template <int dim>
      void div_super (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& int_fetest,
	const FEFaceValuesBase<dim>& ext_fe,
	const FEFaceValuesBase<dim>& ext_fetest);
				       /**
					* Jump of u times normal vector component.
					*/
      template <int dim>
      void u_ni (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const FEFaceValuesBase<dim>& int_fetest,
	const FEFaceValuesBase<dim>& ext_fetest,
	unsigned int i);
   
				       /**
					* Jump of u times normal vector component.
					*/
      template <int dim>
      void u_ni_scaled  (
	FullMatrix<double>& intint,
	FullMatrix<double>& intext,
	FullMatrix<double>& extint,
	FullMatrix<double>& extext,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const FEFaceValuesBase<dim>& int_fetest,
	const FEFaceValuesBase<dim>& ext_fetest,
	unsigned int d,
	const std::vector<double>& int_factor,
	const std::vector<double>& ext_factor);
    
				       /**
					* Jump of u.
					*/
      template <int dim>
      void jump (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const double factor2);
				       /**
					* Jump of u.
					*/
       template <int dim>
      void jump_scaled (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const std::vector<double>& factor,
	const double factor2,
	bool divide = false);
				       /**
					* Flux for the interior
					* penalty method for the
					* Laplacian. With jumping
					* coefficients if @p
					* ext_factor is
					* nonnegative. The @p penalty
					* should be computed for
					* coefficient 1.
					*/
      template <int dim>
      void ip (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	double penalty,
	double int_factor = 1.,
	double ext_factor = -1.);
				       /**
					* Flux for the interior penalty
					* method for the Laplacian with
					* coefficient.
					*/
      template <int dim>
      void ip_scaled (
	FullMatrix<double>& Mii,
	FullMatrix<double>& Mie,
	FullMatrix<double>& Mei,
	FullMatrix<double>& Mee,
	const FEFaceValuesBase<dim>& int_fe,
	const FEFaceValuesBase<dim>& ext_fe,
	const std::vector<double>& factor,
	double penalty);

    }
  }
}

#endif
