//---------------------------------------------------------------------------
//    $Id: random.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __deal2__appframe__random_h
#define __deal2__appframe__random_h

#include <appframe/base.h>

namespace dealii
{
    class ParameterHandler;
    template<typename T> class BlockVector;
}


namespace AppFrame
{
/**
 * An application class solving the same problem with a set of random
 * start vectors.
 *
 * @author Guido Kanschat, 2006
 */
  class Random : public ApplicationCopy
  {
    public:
				       /**
					* This Event is raised
					* whenever Random picked a new
					* value
					*/
      static const Event new_value;
      
				       /**
					* Constructor, receiving the
					* application solving the
					* problem.
					*
					* If #store_vectors is
					* <tt>false</tt>, then call
					* #evaluate() after every
					* sample. Else, store all
					* results in #results and
					* leave the call of
					* #evaluate() to the enclosing
					* function; in this case, you
					* should derive a class with a
					* function #evaluate() in
					* order to have access to
					* #results.
					*/
      Random (ApplicationBase& app,
	      bool store_vectors = false);
      
				       /**
					* Clean up allocated vectors
					* in destructor
					*/
      ~Random ();
      virtual void declare_parameters (ParameterHandler& param);
      
				       /**
					* Read the parameters.
					*/
      void _initialize (ParameterHandler& param);
      
      virtual void initialize (ParameterHandler& param);
      
				       /**
					* The amplitude of the random
					* distribution.
					*/
      double amplitude () const;
      
				       /**
					* Solve the problem a lot of
					* times. If the vectors are
					* not stored (#store_vectors
					* is false), then the function
					* evaluate of the underlying
					* application may accumulate
					* results in a named vector
					* "Random", which will then be
					* returned as the value of
					* <tt>start.</tt>
					*
					* @todo The return value does
					* not differ from the input
					* right now.
					*/
      virtual void solve(FEVector& start, const FEVectors& rhs);
      
    protected:
				       /**
					* A vector storing the results
					* of all the runs. This can be
					* used by the function
					* evaluate() of a derived
					* class to do some statistical
					* analysis of these results.
					*/
      std::vector<SmartPointer<FEVector > > results;
      
				       /**
					* The amplitude of the sample
					* around the start value
					* scaled by the maximum random
					* number.
					*/
      double scaled_amplitude;
				       /**
					* If this is <tt>false</tt>,
					* don't store the vectors
					* after each sample but rather
					* call evaluate.
					*/
      bool store_vectors;
  };
}



#endif
