// ------------------------------------------------------------------
//  $Id: residual.h 1348 2013-08-16 00:45:07Z secanell $
//
//  Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat.
//  Copyright (C) 2012 by Valentin N. Zingan, University of Alberta.
//
//  This file is subject to QPL and may not be  distributed
//  without copyright and license information. Please refer
//  to the file deal.II/doc/license.html for the  text  and
//  further information on this license.
// ------------------------------------------------------------------

#ifndef _DEALII_APPFRAME_RESIDUAL_H_
#define _DEALII_APPFRAME_RESIDUAL_H_

#include <appframe/base.h>

using namespace dealii;

namespace AppFrame
{

/**
 * An abstract class for computing residuals. Its only property is the
 * parenthesis operator, which computes the residual.
 *
 * @author Guido Kanschat, 2008
 */

class Residual : public Subscriptor
{
public:

  virtual ~Residual() { }

  /**
   * This function returns the
   * residual in <tt>dst</tt>,
   * using the vectors from <tt>src</tt>.
   */
  virtual double operator() (FEVector& dst,
                             const FEVectors& src) const = 0;
};

/**
 * A class computing a Residual using the function
 * Application::residual() of the derived Application.
 *
 * @author Guido Kanschat, 2008
 */

class ApplicationResidual : public Residual
{
public:

  /**
   * Constructor initializing
   * this object to compute the
   * standard residual of
   * <tt>application</tt>.
   */
  ApplicationResidual(ApplicationBase& ex_app);

  virtual double operator() (FEVector& dst,
                             const FEVectors& src) const;

private:

  /**
   * The Application class
   * providing the function
   * Application::residual().
   */
  SmartPointer<ApplicationBase> app;
};

inline
ApplicationResidual::ApplicationResidual(ApplicationBase& ex_app)
:
app(&ex_app)
{ }

inline
double
ApplicationResidual::operator() (FEVector& dst,
                                 const FEVectors& src) const
{
  return app->residual(dst, src);
}

}

#endif