//---------------------------------------------------------------------------
//    $Id: runge_kutta.h 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __deal2__appframe__runge_kutta_h
#define __deal2__appframe__runge_kutta_h

#include <appframe/base.h>
#include <appframe/timestep_control.h>

namespace dealii
{
  class ParameterHandler;
  template<typename T> class BlockVector;
}

namespace AppFrame
{
/**
 * Application class for explicit Runge-Kutta methods.
 *
 * @author Guido Kanschat, 2007
 */
  class RungeKuttaExplicit :
      public ApplicationCopy
  {
    public:
  				       /**
					* Constructor, receiving the
					* application computing the
					* residual in each timestep.
					*/
      RungeKuttaExplicit (ApplicationBase& app);
      
				       /**
					* The actual timestep solver.
					*/
      virtual void solve(FEVector& start, const FEVectors& rhs);
      virtual void declare_parameters (ParameterHandler& param);
      virtual void initialize (ParameterHandler& param);
      
				       /**
					* Read the parameters.
					*/
      void _initialize (ParameterHandler& param);

				       /**
					* Set the specific Runge-Kutta
					* scheme.
					*/
      void set_scheme(const std::string name);

				       /**
					* Exception thrown if the
					* string describing the
					* selected scheme is not
					* recognized.
					*/
      DeclException1(ExcUnknownScheme, std::string,
		     << "A scheme with name " << arg1 << " is not known");
      
    protected:
      TimestepControl control;

    private:
				       /**
					* Resize the two vectors to be
					* a triangle that holds all
					* the coefficients of the
					* Runge-Kutta method.
					*/
      void resize_vectors(unsigned int n);
				       /**
					* First coefficient vector.
					*/
      std::vector<std::vector<double> > a;
				       /**
					* Second coefficient vector,
					* denoting the weights when
					* finally summing up the
					* contributions.
					*/
      std::vector<double> b;
				       /**
					* Third coefficient vector,
					* denoting the position in the
					* time interval.
					*/
      std::vector<double> c;

  };
  
}

#endif
