//---------------------------------------------------------------------------
//    $Id: mesh_loop.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <base/parameter_handler.h>
#include <base/logstream.h>
#include <lac/block_vector.h>
#include <appframe/mesh_loop.h>

#include <boost/shared_ptr.hpp>

#include <fstream>
#include <iomanip>

using namespace AppFrame;
using namespace std;


const Event MeshLoop::new_mesh = Event:: assign("Mesh");


MeshLoop::MeshLoop(ApplicationBase& app)
		: ApplicationCopy(app)
{
  deallog << "->MeshLoop";
}


void MeshLoop::declare_parameters(ParameterHandler& param)
{
  app->declare_parameters(param);
    
  param.enter_subsection("Mesh loop");
  ReductionControl::declare_parameters(param);
  param.leave_subsection();
  param.enter_subsection("Output");
  param.declare_entry("Base name", "T", Patterns::Anything());
  param.declare_entry("Intrinsic error", "-", Patterns::Anything());  
  param.leave_subsection();
}


void MeshLoop::initialize(ParameterHandler& param)
{
  app->initialize(param);
  param.enter_subsection("Mesh loop");
  control.parse_parameters(param);
  param.leave_subsection();
  param.enter_subsection("Output");
  base_name = param.get("Base name");
  std::string intrinsic = param.get("Intrinsic error");
  param.leave_subsection();
}


void
MeshLoop::setup(FEVector&)
{}


void
MeshLoop::do_remesh(FEVector& start)
{
	  app->remesh();
	  app->start_vector(start, "MeshLoop");
}


void
MeshLoop::do_evaluate(const FEVectors& src)
{
  app->evaluate(src);
}


void
MeshLoop::solve(FEVector& start, const FEVectors& in_vectors)
{
  FEVector* f = get_data()->block_vector_pool.alloc();
  FEVectors src1;
  FEVectors src2;
  src1.add_vector(start, "MeshLoop");
  src1.merge(in_vectors);
  src2.add_vector(*f, "MeshLoop residual");
  src2.merge(src1);
  
  setup(start);
  
  for (unsigned int iteration = 0; iteration< control.max_steps();++iteration)
    {
      deallog << "MeshLoop::step " << iteration << endl;
      if (iteration > 0)
	do_remesh(start);
      
      char * grid_n = new char[100];
      sprintf(grid_n, "%s.%02d", "grid", iteration);
      app->grid_out(string(grid_n));
      
      f->reinit(start);
      app->residual(*f, src1);
      app->notify(new_mesh);
      try 
	{
	  app->solve(start, src2);
	}
      catch(...) 
	{
	}
	
      do_evaluate(src1);	

      sprintf(grid_n, "%s.%02d", base_name.c_str(), iteration);
      FEVectors outvecs;
      outvecs.add_vector(start, "u");
      data_out(string(grid_n), outvecs);
	
      delete[] grid_n;
	
      double est = app->estimate(src1);
      deallog << "Estimate " << est << endl;
    }
//    if (old != 0)
//      get_data()->block_vector_pool.free(old);
  get_data()->block_vector_pool.free(f);
}


void
MeshLoop::data_out(const std::string& fname, const FEVectors& src)
{
  app->data_out(fname, src);
}


void
MeshLoop::set_name(std::string new_name)
{
  base_name = new_name;
}
