//---------------------------------------------------------------------------
//    $Id: block_matrix_01.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

// Test if subroutines are called with correct arguments

#include <appframe/block_matrix_application.h>
#include <appframe/mesh_loop.h>
#include <base/parameter_handler.h>
#include <lac/block_vector.h>
#include <grid/grid_generator.h>

using namespace dealii;
using namespace AppFrame;

template <int dim>
class TestApplication :
  public BlockMatrixApplication<dim>
{
  public:
    TestApplication ();
    void _initialize();
    virtual void initialize(ParameterHandler& param);
    virtual void dirichlet_bc(std::map<unsigned int, double>& boundary_values) const;
    virtual void solve(FEVector& start, const FEVectors& rhs);
};


template <int dim>
TestApplication<dim>::TestApplication()
{
  deallog << "->BlockMatrix01-" << dim;
  this->cell_couplings.reinit(3, 3);
  this->flux_couplings.reinit(3, 3);
  this->cell_couplings(0,0) = DoFTools::always;
  this->cell_couplings(2,0) = DoFTools::always;
  this->cell_couplings(2,1) = DoFTools::always;
}


template <int dim>
void
TestApplication<dim>::initialize(ParameterHandler& param)
{
  BlockMatrixApplication<dim>::initialize(param);
  _initialize();
}


template <int dim>
void
TestApplication<dim>::_initialize()
{
  GridGenerator::hyper_cube(*this->tr);
  this->remesh_dofs();
  this->remesh_matrices();
}

template <int dim>
void 
TestApplication<dim>::dirichlet_bc(std::map<unsigned int, double>& /*boundary_values*/) const
{}

template <int dim>
void
TestApplication<dim>::solve(FEVector&, const FEVectors& rhs)
{
  if (this->notifications.any())
    {
      this->assemble(rhs);
      this->clear_events();
    }
}


int
main(int argc, char** argv)
{
  AppFrame::initialize(argc, argv);
  AppFrame::open_logfile();
  deallog << "Program:" << argv[0] << std::endl;
  
  ParameterHandler param;
  TestApplication<2> mapp;
  MeshLoop app(mapp);
  deallog << std::endl;

  AppFrame::declare_parameter_files(param);
  app.declare_parameters(param);
  read_parameter_files(param);
  app.initialize(param);
  
  FEVector u;
  FEVectors dummy;
  app.start_vector(u, "Test");
  
  try
    {
      app.solve(u, dummy);
    }
  catch (const std::exception& e)
    {
      deallog << e.what() << std::endl;
    }
}
