//---------------------------------------------------------------------------
//    $Id: dof_01.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#define APPNAME DoF01

// Test number of dofs

#include <appframe/dof_application.h>
#include <appframe/mesh_loop.h>
#include <base/parameter_handler.h>
#include <lac/block_vector.h>
#include <grid/grid_generator.h>

using namespace AppFrame;

template <int dim>
class APPNAME :
  public DoFApplication<dim>
{
  public:
    APPNAME();
    void _initialize();
    virtual void initialize(ParameterHandler& param);
    virtual void cell_residual(FEVector&, const typename DoFApplication<dim>::CellInfo&);
    virtual void bdry_residual(FEVector&, const typename DoFApplication<dim>::FaceInfo&);
    virtual void face_residual(FEVector&, FEVector&,
			       const typename DoFApplication<dim>::FaceInfo&,
			       const typename DoFApplication<dim>::FaceInfo&);
};


template <int dim>
APPNAME<dim>::APPNAME()
{
  deallog << "->DoF01-" << dim;
}


template <int dim>
void
APPNAME<dim>::initialize(ParameterHandler& param)
{
  DoFApplication<dim>::initialize(param);
  _initialize();
}


template <int dim>
void
APPNAME<dim>::_initialize()
{
  GridGenerator::hyper_cube(*this->tr);
  this->remesh_dofs();
  this->block_info.initialize_local(*this->dof);
}


template <int dim>
void
APPNAME<dim>::cell_residual(FEVector&, const typename DoFApplication<dim>::CellInfo&)
{}


template <int dim>
void
APPNAME<dim>::bdry_residual(FEVector&, const typename DoFApplication<dim>::FaceInfo&)
{}


template <int dim>
void
APPNAME<dim>::face_residual(FEVector&,FEVector&,
  const typename DoFApplication<dim>::FaceInfo&, const typename DoFApplication<dim>::FaceInfo&)
{}


int
main(int argc, char** argv)
{
  AppFrame::initialize(argc, argv);
  AppFrame::open_logfile();
  deallog << "Program:" << argv[0] << std::endl;
  
  ParameterHandler param;
  APPNAME<2> tapp;
  MeshLoop app(tapp);
  deallog << std::endl;

  AppFrame::declare_parameter_files(param);
  app.declare_parameters(param);
  read_parameter_files(param);
  app.initialize(param);
  
  FEVector u;
  FEVectors dummy;
  app.start_vector(u, "Test");
  
  try
    {
      app.solve(u, dummy);
    }
  catch (const std::exception& e)
    {
      deallog << e.what() << std::endl;
    }
}
