//---------------------------------------------------------------------------
//    $Id: matrix_01.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#define APPNAME Matrix01

// Test if subroutines are called with correct arguments

#include <appframe/matrix_application.h>
#include <appframe/mesh_loop.h>
#include <base/parameter_handler.h>
#include <lac/block_vector.h>
#include <grid/grid_generator.h>

using namespace AppFrame;

template <int dim>
class APPNAME :
  public MatrixApplication<dim>
{
  public:
    APPNAME ();
    void _initialize();
    virtual void initialize(ParameterHandler& param);
    virtual void solve(FEVector& start, const FEVectors& rhs);
    virtual double evaluate (const FEVectors& src);
};


template <int dim>
APPNAME<dim>::APPNAME()
		:
		MatrixApplication<dim>(true)
{
  deallog << "->Matrix01-" << dim;
  this->cell_couplings.reinit(3, 3);
  this->cell_couplings(0,0) = DoFTools::always;
  this->cell_couplings(2,0) = DoFTools::always;
  this->cell_couplings(2,1) = DoFTools::always;
  this->flux_couplings.reinit(3, 3);

  this->add_matrix(0,0);
  this->add_matrix(2,0);
  this->add_matrix(2,0);
  this->add_matrix(2,1);
  
   this->mg_cell_couplings.reinit(3, 3);
   this->mg_cell_couplings(1,1) = DoFTools::always;
   this->mg_flux_couplings.reinit(3, 3);

   this->add_mg_matrix(1,1);
   this->add_mg_matrix(2,1);
}


template <int dim>
void
APPNAME<dim>::initialize(ParameterHandler& param)
{
  MatrixApplication<dim>::initialize(param);
  _initialize();
}


template <int dim>
void
APPNAME<dim>::_initialize()
{
  GridGenerator::hyper_cube(*this->tr);
  this->remesh_dofs();
  this->remesh_matrices();
  this->verbosity = 0;
}


template <int dim>
void
APPNAME<dim>::solve(FEVector&, const FEVectors& rhs)
{
  if (this->notifications.any())
    {
      this->assemble(rhs);
      this->clear_events();
    }
}


template <int dim>
double
APPNAME<dim>::evaluate(const FEVectors&)
{
  for (unsigned int k=0;k<this->matrices.size();++k)
    deallog << "matrix " << k
	    << ": " << this->matrices[k]->matrix.m()
      	    << "x" << this->matrices[k]->matrix.n()
	    << '\t' << this->matrices[k]->matrix.n_nonzero_elements()
	    << std::endl;
  
  for (unsigned int l=this->mg_matrices[0]->matrix.get_minlevel();
       l<=this->mg_matrices[0]->matrix.get_maxlevel();++l)
    for (unsigned int k=0;k<this->mg_matrices.size();++k)
      deallog << "level " << l
	      << " matrix " << k
	      << ": " << this->mg_matrices[k]->matrix[l].m()
	      << "x" << this->mg_matrices[k]->matrix[l].n()
	      << '\t' << this->mg_matrices[k]->matrix[l].n_nonzero_elements()
	      << std::endl;
  return 1.;
}


int
main(int argc, char** argv)
{
  AppFrame::initialize(argc, argv);
  AppFrame::open_logfile();
  deallog << "Program:" << argv[0] << std::endl;
  
  ParameterHandler param;
  APPNAME<2> mapp;
  MeshLoop app(mapp);
  deallog << std::endl;

  app.declare_parameters(param);
  read_parameter_files(param);
  app.initialize(param);
  
  FEVector u;
  FEVectors dummy;
  app.start_vector(u, "Test");
  
  try
    {
      app.solve(u, dummy);
    }
  catch (const std::exception& e)
    {
      deallog << e.what() << std::endl;
    }
}
