//---------------------------------------------------------------------------
//    $Id: mesh_worker_01.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <base/parameter_handler.h>
#include <lac/block_vector.h>
#include <grid/grid_generator.h>

#include <appframe/dof_application.h>
#include <appframe/mesh_worker.h>
#include <appframe/mesh_worker_loop.h>
#include <appframe/mesh_loop.h>
#include "test_app.h"


  template <int dim>
  class LocalWorker
  {
    public:
				       /**
					* Do the work on a cell.
					*/
      void cell(const MeshWorker::InfoObjects::DoFInfo<dim>& info)
	{
	  deallog << "Cell " << info.cell->center() << std::endl;
	}
      
      
				       /**
					* Do the work on a boundary face.
					*/
      void bdry(const MeshWorker::InfoObjects::DoFInfo<dim>& info)
	{
	  deallog << "Bdry " << info.face_number << ' ' << info.face->center() << std::endl;
	}
      
				       /**
					* Do the work on an interior face.
					*/
      void face(const MeshWorker::InfoObjects::DoFInfo<dim>& info1,
		const MeshWorker::InfoObjects::DoFInfo<dim>& info2)
	{
	  deallog << "Face " << info1.face_number << ' ' << info1.face->center() << std::endl;
	  deallog << "Ngbr " << info2.face_number << ' ' << info2.face->center() << std::endl;
	}

				       /**
					* Computations on interior
					* faces are necessary.
					*/
      bool interior_fluxes;
      
				       /**
					* Computations on interior
					* faces are necessary.
					*/
      bool boundary_fluxes;
      
  };



template<int dim>
class MeshWorker01 :
  public DoFApplication<dim>
{
    virtual void initialize(ParameterHandler& param)
      {
	DoFApplication<dim>::initialize(param);
	GridGenerator::hyper_cube(*this->tr, -16, 16);
	this->remesh_dofs();
	this->block_info.initialize_local(*this->dof);
      }

    virtual double residual(FEVector&, const FEVectors&) 
      { return 1.; }

    virtual void solve(FEVector&, const FEVectors&src)
      {
	MeshWorker::InfoObjects::DoFInfo<dim> ci(src, this->block_info);
	MeshWorker::InfoObjects::DoFInfo<dim> bi(src, this->block_info);
	MeshWorker::InfoObjects::DoFInfo<dim> fi(src, this->block_info);
	MeshWorker::InfoObjects::DoFInfo<dim> ni(src, this->block_info);
	
	LocalWorker<dim> local;
	local.boundary_fluxes = true;
	local.interior_fluxes = true;
	
	MeshWorker::loop(this->dof->begin_active(), this->dof->end(),
			 ci, bi, fi, fi, ni, local);
      }
};


int
main(int argc, char** argv)
{
  AppFrame::initialize(argc, argv);
  AppFrame::open_logfile();
  deallog << "Program:" << argv[0] << std::endl;
  
  ParameterHandler param;
  MeshWorker01<2> dapp;
  MeshLoop app(dapp);
  deallog << std::endl;

  AppFrame::declare_parameter_files(param);
  app.declare_parameters(param);
  AppFrame::read_parameter_files(param);
  app.initialize(param);
  
  FEVector u;
  FEVectors dummy;
  app.start_vector(u, "mesh_loop");
  
  try
    {
      app.solve(u, dummy);
    }
  catch (const std::exception& e)
    {
      deallog << e.what() << std::endl;
    }
}
