//---------------------------------------------------------------------------
//    $Id: newton.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007, 2008 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <appframe/newton.h>
#include <base/parameter_handler.h>
#include <lac/block_vector.h>
#include "test_app.h"

class TestResidual : public AppFrame::Residual
{
  public:
    virtual double operator() (FEVector& dst, const FEVectors& src) const;
};


double
TestResidual::operator() (FEVector& dst, const FEVectors& src) const
{
  const double u = src.vector(0).block(0)(0);
  const double r = u*u-3.;
  dst.block(0)(0) = r;
  return std::fabs(r);
}


int
main(int argc, char** argv)
{
  AppFrame::initialize(argc, argv);
  AppFrame::open_logfile();
  deallog << "Program:" << argv[0] << std::endl;
  
  ParameterHandler param;
  SquareApplication dapp;
  Newton app(dapp);
  TestResidual r3;
  Newton app3(dapp, r3);
  deallog << std::endl;

  app.declare_parameters(param);
  read_parameter_files(param);
  app.initialize(param);
  app3.initialize(param);
  
  FEVector u;
  FEVectors dummy;

  
  try
    {
      app.start_vector(u, "Newton");
      app.solve(u, dummy);
      deallog << "sqrt(2) = " << u.block(0)(0) << std::endl;
      dapp.notify (Newton::bad_derivative);
      app3.start_vector(u, "Newton");      
      app3.solve(u, dummy);
      deallog << "sqrt(3) = " << u.block(0)(0) << std::endl;
    }
  catch (const std::exception& e)
    {
      deallog << e.what() << std::endl;
    }
}
