//---------------------------------------------------------------------------
//    $Id: random_01.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2006, 2007 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <appframe/random.h>
#include <base/parameter_handler.h>
#include <lac/block_vector.h>
#include "test_app.h"

#include <cmath>


class RandomTest :
  public AppFrame::Random
{
  public:
				     /**
				      * Initialize application with
				      * storing vectors.
				      */
    RandomTest(ApplicationBase& app);
				     /**
				      * Compute statistical values
				      * from the results.
				      */
    double evaluate(const FEVectors&);
};


RandomTest::RandomTest(ApplicationBase& app)
		:
		AppFrame::Random(app, true)
{}


double
RandomTest::evaluate(const FEVectors&)
{
  deallog << "Evaluate ";
  double average = 0.;
  double deviation = 0.;
  for (unsigned int i=0;i<results.size();++i)
    average += results[i]->block(0)(0);
  average /= results.size();
  for (unsigned int i=0;i<results.size();++i)
    deviation += std::pow(results[i]->block(0)(0) - average, 2.);
  deviation = std::sqrt(deviation/results.size());
  
  deallog << "average: " << average
	  << "  deviation " << deviation
	  << std::endl;
  return average;
}


int
main(int argc, char** argv)
{
  AppFrame::initialize(argc, argv);
  AppFrame::open_logfile();
  deallog << "Program:" << argv[0] << std::endl;
  
  ParameterHandler param;
  RandomApplication dapp;
  RandomTest app(dapp);
  deallog << std::endl;

  AppFrame::declare_parameter_files(param);
  app.declare_parameters(param);
  read_parameter_files(param);
  app.initialize(param);
  
  FEVector u;
  FEVectors dummy;
  dummy.add_vector(u, "RandomTest");
  app.start_vector(u, "Random");
  
  try
    {
      app.solve(u, dummy);
    }
  catch (const std::exception& e)
    {
      deallog << e.what() << std::endl;
    }
  app.evaluate(dummy);
}
