//---------------------------------------------------------------------------
//    $Id: shop_bdry_01.cc 1348 2013-08-16 00:45:07Z secanell $
//
//    Copyright (C) 2007 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

// Test cell matrices and residuals for grad and div operators

#include <appframe/residual_shop_bdry.h>
#include <appframe/matrix_shop_bdry.h>
#include <appframe/matrix_base.h>
#include <base/quadrature_lib.h>
#include <lac/block_vector.h>
#include <grid/grid_generator.h>
#include <grid/tria_iterator.h>
#include <grid/tria_accessor.h>
#include <fe/fe_dgp.h>
#include <fe/fe_q.h>
#include <fe/fe_raviart_thomas.h>

#include <iostream>

using namespace AppFrame;

FullMatrix<double> matrix;

template<int dim>
void u_dot_n(const FEFaceValuesBase<dim>& fe,
	 const FEFaceValuesBase<dim>& fetest)
{
  deallog.push("u_dot_n");  
  const unsigned int n_dofs = fe.get_fe().dofs_per_cell;
  const unsigned int t_dofs = fetest.get_fe().dofs_per_cell;
  std::vector<double> factor(fe.n_quadrature_points, 2.);
  
				   // Index vector needed for
				   // get_function_values etc.
  std::vector<unsigned int> indices(n_dofs*dim/fe.get_fe().n_components());
  for (unsigned int i=0;i<indices.size();++i)
    indices[i] = i;

				   // The local finite element vector
  Vector<double> function(indices.size());
  Vector<double> result(fetest.get_fe().dofs_per_cell);

  std::vector<std::vector<double> >
    values(dim, std::vector<double>(fe.n_quadrature_points));
  
  deallog << "matrix" << std::endl;
  std::vector<MatrixBlock<FullMatrix<double> > > M(dim);
  std::vector<unsigned int> Mi(dim/fe.get_fe().n_components());
  for (unsigned int d=0;d<dim;++d)
    {
      Mi[d] = d;
      M[d].column = d;
      M[d].matrix.reinit(t_dofs, n_dofs);
    }
  AppShop::Matrix::Boundary::u_dot_n(M, Mi, fe, fetest);
  matrix.reinit(t_dofs, function.size());
  for (unsigned int d=0;d<Mi.size();++d)
    for (unsigned int i=0;i<t_dofs;++i)
      {
	for (unsigned int j=0;j<n_dofs;++j)
	  {
	    matrix(i, j+d*n_dofs) = M[d].matrix(i,j);
	    deallog << ' ' << M[d].matrix(i,j);
	  }
	deallog << std::endl;
      }
  
  deallog << "scaled matrix consistency";
  for (unsigned int d=0;d<dim;++d)
    M[d].matrix.reinit(t_dofs, n_dofs);
  AppShop::Matrix::Boundary::u_dot_n_scaled(M, Mi, fe, fetest, factor, true);
  for (unsigned int d=0;d<Mi.size();++d)
    for (unsigned int i=0;i<t_dofs;++i)
      for (unsigned int j=0;j<n_dofs;++j)
	if (std::fabs(M[d].matrix(i,j) - .5 * matrix(i,j+d*n_dofs)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ';' << (M[d].matrix(i,j)) << ']';
  AppShop::Matrix::Boundary::u_dot_n_scaled(M, Mi, fe, fetest, factor, false);
  for (unsigned int d=0;d<Mi.size();++d)
    for (unsigned int i=0;i<t_dofs;++i)
      for (unsigned int j=0;j<n_dofs;++j)
	if (std::fabs(M[d].matrix(i,j) - 2.5 * matrix(i,j+d*n_dofs)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ';' << (M[d].matrix(i,j)) << ']';
  AppShop::Matrix::Boundary::u_dot_n(M, Mi, fe, fetest, -1.5);
  for (unsigned int d=0;d<Mi.size();++d)
    for (unsigned int i=0;i<t_dofs;++i)
      for (unsigned int j=0;j<n_dofs;++j)
	if (std::fabs(M[d].matrix(i,j) - matrix(i,j+d*n_dofs)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ';' << (M[d].matrix(i,j)) << ']';
  deallog << std::endl;
  
  deallog << "residual consistency";  
  for (unsigned int i=0;i<function.size();++i)
    {
      result = 0.;
      function = 0.;
      function(i) = 1.;
      fe.get_function_values(function, indices, values, true);
				       // Test with default factor 1
      AppShop::Residual::Boundary::u_dot_n(result, values, fe, fetest);
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - matrix(j,i)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ']';
				       // Add same with factor .5
      AppShop::Residual::Boundary::u_dot_n(result, values, fe, fetest, .5);
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - 1.5 * matrix(j,i)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ']';
    }
  deallog << std::endl;
  
  deallog << "scaled residual consistency";
  for (unsigned int i=0;i<function.size();++i)
    {
      function = 0.;
      function(i) = 1.;
      fe.get_function_values(function, indices, values, true);
      result = 0.;
      AppShop::Residual::Boundary::u_dot_n_scaled(result, values, fe, fetest, factor, false);
      result *= .5;
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - matrix(j,i)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ';' << result(j) << ']';
      result = 0.;
      AppShop::Residual::Boundary::u_dot_n_scaled(result, values, fe, fetest, factor, true);
      result *= 2.;
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - matrix(j,i)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ':' << result(j) << ']';
    }
  deallog << std::endl;
  
  deallog.pop();
}


template<int dim>
void u_times_n(const FEFaceValuesBase<dim>& fe,
	  const FEFaceValuesBase<dim>& fetest)
{
  deallog.push("u_times_n");
  const unsigned int n_dofs = fe.get_fe().dofs_per_cell;
  const unsigned int t_dofs = fetest.get_fe().dofs_per_cell;
  
				   // Index vector needed for
				   // get_function_values etc.
  std::vector<unsigned int> indices(n_dofs);
  for (unsigned int i=0;i<n_dofs;++i)
    indices[i] = i;

				   // The local finite element vector
  Vector<double> function(n_dofs);
  Vector<double> result(t_dofs * dim/fetest.get_fe().n_components());
  
  std::vector<double> values(fe.n_quadrature_points);  
  
  deallog << "residual consistency";
  for (unsigned int i=0;i<n_dofs;++i)
    {
      result = 0.;
      function = 0.;
      function(i) = 1.;
      fe.get_function_values(function, indices, values);
      AppShop::Residual::Boundary::u_times_n(result, values, fe, fetest);
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - matrix(i,j)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ']';
      AppShop::Residual::Boundary::u_times_n(result, values, fe, fetest, .5);
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - 1.5 * matrix(i,j)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ']';
    }
  deallog << std::endl;

  deallog << "scaled residual consistency";
  std::vector<double> factor(fe.n_quadrature_points, 2.);
  for (unsigned int i=0;i<n_dofs;++i)
    {
      function = 0.;
      function(i) = 1.;
      fe.get_function_values(function, indices, values);
      result = 0.;
      AppShop::Residual::Boundary::u_times_n_scaled(result, values, fe, fetest, factor, false);
      result *= .5;
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - matrix(i,j)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ']';
      result = 0.;
      AppShop::Residual::Boundary::u_times_n_scaled(result, values, fe, fetest, factor, true);
      result *= 2.;
      for (unsigned int j=0;j<result.size();++j)
	if (std::fabs(result(j) - matrix(i,j)) > 1.e-14)
	  deallog << '[' << i << ',' << j << ']';
    }
  deallog << std::endl;

  deallog.pop();
}


template<int dim>
void test_element(const typename Triangulation<dim>::cell_iterator cell,
		  FiniteElement<dim>& el,
		  FiniteElement<dim>& el_test)
{
  QGauss<dim-1> face_quadrature(el.degree+1);
  FEFaceValues<dim> fe(el, face_quadrature,
		       UpdateFlags(update_values
				   | update_JxW_values
				   | update_normal_vectors));
  FEFaceValues<dim> fetest(el_test, face_quadrature,
			    UpdateFlags(update_values
					| update_JxW_values
					| update_normal_vectors));
  
  deallog << fe.get_fe().get_name() << " <-> "
	  << fetest.get_fe().get_name() << std::endl;
  
  for (unsigned int face=0;face<GeometryInfo<dim>::faces_per_cell;++face)
    {
      fe.reinit(cell, face);
      fetest.reinit(cell, face);
      u_dot_n(fetest, fe);
      u_times_n(fe, fetest);
    }
}


template<int dim>
void test()
{
  deallog.threshold_double(1.e-12);
  
  Triangulation<dim> tr;
  GridGenerator::hyper_cube(tr, 0, 1);
  
  FE_Q<dim> q1(1);
  FE_Q<dim> q2(2);
  
  FE_DGP<dim> dgp1(1);

  FE_RaviartThomas<dim> rt1(1);
  
  test_element(tr.begin_active(), dgp1, dgp1);
  test_element(tr.begin_active(), q1, q1);
  test_element(tr.begin_active(), q2, q1);
  test_element(tr.begin_active(), q1, rt1);
}


int main()
{
  test<2>();
}
