//---------------------------------------------------------------------------
//    $Id: test_app.h 1348 2013-08-16 00:45:07Z secanell $
//    Version: $Name:  $
//
//    Copyright (C) 2006, 2007, 2008, 2009 by Guido Kanschat
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <appframe/base.h>

using namespace AppFrame;

/**
 * Application with each function just printing its name.
 */
class DummyApplication
  :
  public ApplicationBase
{
  public:
    DummyApplication();
    virtual void declare_parameters(ParameterHandler& param);
    virtual void initialize (ParameterHandler& param);
    virtual void remesh();
    virtual void init_vector (FEVector& dst) const;
    virtual void start_vector (FEVector& dst,
			       std::string) const;
    
    virtual double residual(FEVector& dst, const FEVectors& src);
    virtual void solve(FEVector& start, const FEVectors& rhs);
    virtual void Tsolve(FEVector& start, const FEVectors& rhs);
				     /**
				      * Error estimation
				      */
    virtual double estimate(const FEVectors& src);
    virtual double evaluate(const FEVectors& src);
    virtual void grid_out(const std::string& filename) const;
    virtual void data_out(const std::string& filename,
			  const FEVectors& src);
    
    virtual void set_time (double t);
    virtual void set_time_step (double t);
    virtual bool autonomous () const;
  private:
    int count;
};


/**
 * Application with only #start_vector() implemented
 */
class RandomApplication
  :
  public ApplicationBase
{
  public:
    RandomApplication();
    virtual void declare_parameters(ParameterHandler& param);
    virtual void init_vector (FEVector& dst) const;
    virtual void start_vector (FEVector& dst,
			       std::string) const;
    virtual void solve(FEVector&, const FEVectors&);
    virtual double residual(FEVector&, const FEVectors&);
    virtual double evaluate(const FEVectors& src);
};


/**
 * Application for computing the square root of two.
 */
class SquareApplication
  :
  public ApplicationBase
{
  public:
    SquareApplication();
    virtual void declare_parameters(ParameterHandler& param);
    virtual void initialize (ParameterHandler& param);
    virtual void remesh();
    virtual void init_vector (FEVector& dst) const;
    virtual void start_vector (FEVector& dst, std::string) const;
    virtual double residual(FEVector& dst, const FEVectors& src);
    virtual void solve(FEVector& start, const FEVectors& rhs);
    virtual double evaluate(const FEVectors& src);
  private:
    double target;
    double derivative;
};


/**
 * Application for ordinary differential equations. The solutions are
 * polynomials.
 */
class ODEApplication
  :
  public ApplicationBase
{
  public:
    ODEApplication();
    virtual void declare_parameters(ParameterHandler& param);
    virtual void initialize (ParameterHandler& param);
    virtual void remesh();
    virtual void init_vector (FEVector& dst) const;
    virtual void start_vector (FEVector& dst, std::string) const;
    virtual double residual(FEVector& dst, const FEVectors& src);
    virtual void solve(FEVector& start, const FEVectors& rhs);
    virtual double evaluate(const FEVectors& src);
};


