//---------------------------------------------------------------------------
//    $Id: MixtureDynamicViscosity.h 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2010 by Ali MalekpourKoupaei
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
#ifndef _FUELCELLSHOP__MixtureDynamicViscosity__H
#define _FUELCELLSHOP__MixtureDynamicViscosity__H

#define __Del(p) {if(p) delete p; p=__null;}
#define __printf __builtin_printf
#define __scanf  __builtin_scanf
#define __exp    __builtin_exp
#define __pow    __builtin_pow
#define __DUMMY  -9999

#include<PureGas.h>

enum enDynamicViscosityModelID {vmChapmanEnskog=1, vmNone};

namespace FuelCellShop
{
  namespace Mixture
  {

  /**
     MixtureDynamicViscosity as an abstract class
  */
  class MixtureDynamicViscosity
  {
  protected:
    FuelCellShop::Material::PureGas *species1, *species2;
  public:
    
    //class constructors
    MixtureDynamicViscosity();
    MixtureDynamicViscosity(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2);
    
    //class distructor
    virtual ~MixtureDynamicViscosity();
    
    // the method's specs.
    virtual char* get_name()			       	               =0;
    virtual char* get_description()		       	               =0;
    virtual enDynamicViscosityModelID get_ID()            	               =0;
    
    virtual void get_Mu (double& Mu)		  	       =0;
    virtual void get_Mu (double& Mu, const double& T, const std::vector<double>& xi) =0;

    virtual void get_dMu_dxi (double& dMu, const double& T, const std::vector<double>& xi) 	   =0;
    virtual void get_dMu_dT (double& dMu, const double& T, const std::vector<double>& xi) 	   =0;
    
    void set_T_n_xi(const double& T, const std::vector<double>& xi);
    void set_species(FuelCellShop::Material::PureGas& species1, 
                     FuelCellShop::Material::PureGas& species2);
    
    virtual void scan();
    virtual void report() ;
    virtual void report2file(char* filename);
    
  protected:
    double T;
    std::vector<double> xi;
  };
  //---------------------------------------------------------------------------
  
  /*
    VaporMixtureDynamicViscosity as a publicly derived class of MixtureDynamicViscosity
  */
  class VaporMixtureDynamicViscosity : public MixtureDynamicViscosity
  {
  public:
    
    //class constructors
    VaporMixtureDynamicViscosity();
    VaporMixtureDynamicViscosity(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2);
    
    //class distructor
    virtual ~VaporMixtureDynamicViscosity();
    
    virtual void scan();
    virtual void report() ;
    virtual void report2file(char* filename);
  };
  //---------------------------------------------------------------------------
  
  /*
    ChapmanEnskogViscosityModel as a publicly derived class of VaporMixtureDynamicViscosity
  */
  class ChapmanEnskogViscosityModel : public VaporMixtureDynamicViscosity
  {
  public:
    
    //class constructors
    ChapmanEnskogViscosityModel();
    ChapmanEnskogViscosityModel(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2);
    
    //class distructor
    virtual ~ChapmanEnskogViscosityModel();
    
    virtual char* get_name();
    virtual char* get_description();
    virtual enDynamicViscosityModelID get_ID();
    
    virtual void get_Mu (double& Mu);
    virtual void get_Mu (double& Mu, const double& T, const std::vector<double>& xi);

    virtual void get_dMu_dxi (double& dMu, const double& T, const std::vector<double>& xi);
    virtual void get_dMu_dT (double& dMu, const double& T, const std::vector<double>& xi);

    void get_exi(std::vector< std::vector<double> >& exi, const double& T);
    
    virtual void scan();
    virtual void report() ;
    virtual void report2file(char* filename);
  };
  //---------------------------------------------------------------------------
  }
}

#endif
