//---------------------------------------------------------------------------
//    $Id: newton_w_3ppC.h 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2009 by Jason Boisvert
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __deal2__appframe__newton_w_3ppC_h
#define __deal2__appframe__newton_w_3ppC_h

#include <appframe/base.h>
#include <appframe/residual.h>
#include <lac/solver_control.h>
#include <newton_base.h>


namespace AppFrame
{
/**
 * Application class performing Newton's iteration. At each iteration a line search is performed using
 * XX algorithm to enhance robustness of the algorithm and to improve convergence to a global solution.
 * This class is based on the class Newton on AppFrame developed by Guido Kanschat
 *
 * @author Jason Boisvert, 2009
 */

class Newton3ppC : public newtonBase
  {
    public:

	static const AppFrame::Event bad_derivative; 

	Newton3ppC(ApplicationBase& app);

	virtual void solve(AppFrame::FEVector& u,
			   const AppFrame::FEVectors& in_vectors);

  private:

	double sum_of_squares(double residual_norm);
	
	/**
	* Merit Function
	*/
	double three_point_step (int line_search_iterations,double lambda_current, double lambda_previous, double mf_original, double mf_current, double mf_previous);
	
	/**
	* determines step size by three point parabolic function
	*/
	bool armijo(double step_length,double residual_norm,double new_residual_norm);

	/**
	* Convergance criterion
	*/
	double max_reduction_factor;
	double min_reduction_factor;
	double alpha;
	int step;
       };
}

#endif
