//---------------------------------------------------------------------------
//    $Id: newton_w_line_search.h 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2007 by Marc Secanell
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#ifndef __deal2__appframe__newton_w_lines_search_h
#define __deal2__appframe__newton_w_lines_search_h

#include <appframe/base.h>
#include <appframe/residual.h>
#include <lac/solver_control.h>
#include <newton_base.h>



namespace AppFrame
{
/**
 * Application class performing Newton's iteration. At each iteration a line search is performed using
 * XX algorithm to enhance robustness of the algorithm and to improve convergence to a global solution.
 * This class is based on the class Newton on AppFrame developed by Guido Kanschat
 *
 * @author Marc Secanell, 2007
 */
  class NewtonLineSearch : public newtonBase
  {
  public:
        /**
	 * The Event set by NewtonLineSearch if
	 * convergence is becoming bad
	 * and a new matrix should be
	 * assembled.
	 */
        static const AppFrame::Event bad_derivative; 
        /**
	 * Constructor, receiving the application computing the residual and
	 * solving the linear problem.
	 */
	NewtonLineSearch (ApplicationBase& app);

	/** Declare the input parameters. */
	virtual void declare_parameters (ParameterHandler& param);

	/** Read the parameters local to NewtonLineSearch. */
	void _initialize (ParameterHandler& param);

	/** Read the parameters */		
	virtual void initialize (ParameterHandler& param);

	/**
	 * The actual Newton solver.
	 */
	virtual void solve(AppFrame::FEVector& u,
			   const AppFrame::FEVectors& in_vectors);


    private:

	/**
	 * Value used to multiply \f$\delta u\f$, the solution update, from the Newton algorithm
	 * during the first overrelax_steps iterations. We will multiply the first
	 * iteration by \f$\alpha \delta u\f$, where \f$ \alpha \f$ is the value of overrelax and then
	 * the following iterations by \f$it \alpha \delta u\f$ where \f$ it \f$ is the iteration number
	 */
	double overrelax;
	/**
	 * Number of iterations to which we apply overrelax
	 */
	unsigned int overrelax_steps;
	/**
	 * Do you want to use a line search at each step?
	 */
	bool line_search;
    };
}

#endif
