// ------------------------------------------------------------------
//
//    $Id: fcst_constants.h 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2009 by Marc Secanell, University of Alberta.
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
// ------------------------------------------------------------------

#ifndef _FCST_CONSTANTS_H_
#define _FCST_CONSTANTS_H_

#include <deal.II/base/tensor.h>
#include <deal.II/base/symmetric_tensor.h>

const unsigned int dim = deal_II_dimension;

namespace Constants
{
      /** Universal gas constant = 8.314462176 J/(mol K)   **/
      extern inline double R()  { return 8.314462176;     }

      /** Faraday constant = 96483.399242424 C/mol         **/
      extern inline double F()  { return 96483.399242424; }

      /** Pi = 3.141592654                                 **/
      extern inline double Pi() { return 3.141592654;     }

      /** Permittivity of free space = 8.854187818e-12 F/m **/
      extern inline double E0() { return 8.854187818e-12; }

      /** Boltzmann constant = 8.617332478e-5 eV/K         **/
      extern inline double K()  { return 8.617332478e-5;  }

      /**
       * Gravitational acceleration, \f$ \mathbf{g} = \{ g_{\alpha} \}_{\alpha = 1}^d \quad \text{such that} \quad \forall \alpha \neq d : \quad g_{\alpha} = 0 \quad \left[ \frac{\text{m}}{\text{sec}^2} \right] \quad
       * \text{and} \quad g_d = -9.81 \quad \left[ \frac{\text{m}}{\text{sec}^2} \right] \f$.
       */
      extern inline const dealii::Tensor<1,dim> gravity_acceleration()
      {
             dealii::Tensor<1,dim> result;
             result[dim-1] = -9.81;
             return result;
      }

      /**
       * Unit tensor, \f$ \hat{ \mathbf{I} } = \{ \delta_{\alpha \beta} \}_{\alpha,\beta = 1}^d \f$.
       */
      extern inline const dealii::SymmetricTensor<2,dim> unit_tensor()
      {
             dealii::SymmetricTensor<2,dim> result;

             for(unsigned int i = 0; i < dim; ++i)
                    for(unsigned int j = 0; j < dim; ++j)
                           if( i == j )
                                  result[i][j] = 1.0;

             return result;
      }
}

#endif