//---------------------------------------------------------------------------
//    $Id: optimization_block_matrix_application.cc 1389 2013-08-27 23:29:33Z zingan $
//
//    Copyright (C) 2006, 2007 by Marc Secanell
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include "optimization_block_matrix_application.h"

namespace AppFrame
{

template <int dim>
const AppFrame::Event OptimizationBlockMatrixApplication<dim>::sensitivity_analysis = AppFrame::Event::assign("Sensitivity Analysis");

//---------------------------------------------------------------------------
template <int dim>
void
LocalResponse<dim>::cell(const typename AppFrame::DoFApplication<dim>::CellInfo& cell_info)
  {
    // get the functional that I would like to fill
    AppFrame::FEVector marc;
    app->cell_responses (J1, cell_info, marc);
    this->assemble(cell_info);
  }

template <int dim>
void
LocalResponse<dim>::bdry(const typename AppFrame::DoFApplication<dim>::FaceInfo& )
  {}

template <int dim>
void
LocalResponse<dim>::face(const typename AppFrame::DoFApplication<dim>::FaceInfo& ,
                          const typename AppFrame::DoFApplication<dim>::FaceInfo& )
  {}


//---------------------------------------------------------------------------
//---------------- OPTIMIZATIONBLOCKMATRIXAPPLICATION -----------------------
//---------------------------------------------------------------------------
template <int dim>
OptimizationBlockMatrixApplication<dim>::OptimizationBlockMatrixApplication(boost::shared_ptr<AppFrame::ApplicationData> data)
  :
  AppFrame::BlockMatrixApplication<dim>(data)
{
  deallog << "->OptimizationBlockMatrix"<<std::endl;
  optimization = false;
}

//---------------------------------------------------------------------------
template <int dim>
OptimizationBlockMatrixApplication<dim>::OptimizationBlockMatrixApplication(AppFrame::DoFApplication<dim>& other,
									    bool triangulation_only)
  :
  AppFrame::BlockMatrixApplication<dim>(other, triangulation_only)
{
  optimization = false;
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::declare_parameters(ParameterHandler& param)
{
	param.enter_subsection("Output Variables");
	{
		param.declare_entry("num_output_vars",
						"1",
				  Patterns::Integer());

		const unsigned int num_output_vars = 100;
		for (unsigned int index=0; index<num_output_vars; ++index)
		{
			std::ostringstream streamOut;
			streamOut << index;
			std::string name = "Output_var_" + streamOut.str();
			param.declare_entry(name.c_str(),
							"no_name",
				   Patterns::Anything());
		}
	}
	param.leave_subsection();

  AppFrame::BlockMatrixApplication<dim>::declare_parameters(param);
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::set_parameters(const std::vector<std::string>& ,
                                                        const std::vector<double>& ,
                                                        ParameterHandler&)
{
    // Print info:
    const std::type_info& info = typeid(*this);
    std::cout << "Pure function " << __FUNCTION__
    << " called in Class "
    << info.name() << std::endl;

    // Throw an exception:
    ExcNotImplemented ();
}


//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::initialize(ParameterHandler& param)
{
	param.enter_subsection("Output Variables");
		unsigned int num_output_vars = param.get_integer("num_output_vars");
		name_output_var.clear();
		name_output_var.resize(num_output_vars);
		for (unsigned int index=0; index<num_output_vars; ++index)
		{
			std::ostringstream streamOut;
			streamOut << index;
			std::string name = "Output_var_" + streamOut.str();
			name_output_var[index] = param.get(name.c_str());
		}
	param.leave_subsection();

	if(!optimization)
	{
		n_resp = num_output_vars;
		name_responses.clear();
		name_responses = name_output_var;
	}

  AppFrame::BlockMatrixApplication<dim>::initialize(param);
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::responses (std::vector<double>& respon,
													const AppFrame::FEVectors& src)
{
  LocalResponse<dim> integrator;
  integrator.initialize(this, respon.size());
  integrator.boundary_fluxes = false;
  integrator.interior_fluxes = false;

  UpdateFlags update_flags = update_quadrature_points | update_values | update_gradients;
  integrator.add_update_flags(update_flags, true, true, true, true);
   MeshWorker::VectorSelector tmp;

//TODO: Replace this!!!
  for (unsigned int i=0;i<src.n_vectors();++i)
    tmp.add_vector(src.vector_name(i), true, true, false);
  integrator.initialize_selectors(tmp, tmp, tmp);
  integrator.cell_quadrature = this->quadrature_residual_cell;
  integrator.bdry_quadrature = this->quadrature_residual_bdry;
  integrator.face_quadrature = this->quadrature_residual_face;

  MeshWorker::InfoObjects::IntegrationInfoBox<dim> box(this->block_info);
  box.initialize(integrator, *this->element, *this->mapping, src);

  MeshWorker::integration_loop(this->dof->begin_active(), this->dof->end(), box, integrator);

  for (unsigned int i=0;i<respon.size();++i)
	respon[i] = integrator(i);

  global_responses(respon,
		   src.vector(0));

}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::check_responses()
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::cell_responses(std::vector<double>& ,
							const typename DoFApplication<dim>::CellInfo& ,
							const AppFrame::FEVector& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::global_responses (std::vector<double>& ,
							   const AppFrame::FEVector& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::print_responses (std::vector<double>& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::dresponses_dl (std::vector<std::vector<double> >& drespon_dl,
							const AppFrame::FEVectors& src)
{
  /*
	Assert(drespon_dl.size() == n_resp,
		ExcDimensionMismatch(drespon_dl.size(), n_resp));
	Assert(drespon_dl[0].size() == n_dvar,
		ExcDimensionMismatch(drespon_dl[0].size(), n_dvar));

//	Assert (this->quadrature_residual_cell, ExcNotInitialized());
	const Quadrature<dim>& cell_quadrature = this->quadrature_residual_cell;

	// Compute the degrees of freedom per cell:
	//const unsigned int n_dofs = this->element->dofs_per_cell;
	//const unsigned int n_blocks = this->element->n_blocks();
	//const unsigned int n_components = this->element->n_components();

	// Objects for cell and face data
	// handed down to the local
	// routines.
	FEValues<dim>* fevalues = 0;
	typename DoFApplication<dim>::CellInfo cell_info(src, this->block_info);
	cell_info.initialize(fevalues, *this->element, *this->mapping,
			cell_quadrature,
			UpdateFlags(update_q_points
					| update_values
					| update_gradients
					| update_JxW_values), true);

	// Initialize response vector:
	for (unsigned int i=0; i<drespon_dl.size(); ++i)
		for (unsigned int j=0; j<drespon_dl[i].size(); ++j)
			drespon_dl[i][j] = 0.0;

	this->tr->clear_user_flags ();

	typename DoFHandler<dim>::active_cell_iterator c;
	for (c = this->dof->begin_active();
	c != this->dof->end();
	++c)
	{
				// Initialize local structures
		cell_info.reinit(c);

				// Fill local data vectors
		cell_info.fill_local_data(cell_info.values, true);
		cell_info.fill_local_data(cell_info.derivatives, true);

				// Compute dresp_dl
		cell_dresponses_dl(drespon_dl,
					cell_info,
					src.vector(0));
	} // cells

	global_dresponses_dl(drespon_dl,
			src.vector(0));
  */
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::cell_dresponses_dl(std::vector<std::vector<double> >& ,
							    const typename DoFApplication<dim>::CellInfo& ,
							    const AppFrame::FEVector& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::global_dresponses_dl(std::vector<std::vector<double> >& ,
							      const AppFrame::FEVector& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::dresponses_du(std::vector<AppFrame::FEVector >& df_du,
						       const AppFrame::FEVectors& sol)
{
  bool dresponses_du = true;
  bool dresidual_dlambda = false;
  this->dfunction(df_du, sol, dresponses_du, dresidual_dlambda);

  global_dresponses_du(df_du,
		       sol.vector(0));
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::cell_dresponses_du(std::vector<AppFrame::FEVector >& ,
							    const typename DoFApplication<dim>::CellInfo& ,
							    std::vector<std::vector<double> >& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::global_dresponses_du(std::vector<AppFrame::FEVector >& ,
							      const AppFrame::FEVector& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::dresidual_dlambda(
	std::vector<AppFrame::FEVector >& dst,
	const AppFrame::FEVectors& src)
  {
    bool dresponses_du = false;
    bool dresidual_dlambda = true;
    this->dfunction(dst, src, dresponses_du, dresidual_dlambda);
  }

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::cell_dresidual_dlambda(
	std::vector<AppFrame::FEVector >& ,
	const typename DoFApplication<dim>::CellInfo& ,
	std::vector<std::vector<double> >& )
{
  // Print info:
  const std::type_info& info = typeid(*this);
  std::cout << "Pure function " << __FUNCTION__
	    << " called in Class "
	    << info.name() << std::endl;

  // Throw an exception:
  ExcNotImplemented ();
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::solve_direct (std::vector<std::vector<double> >& df_dl,
						       const AppFrame::FEVectors& vectors)
{

  // Assertations
  Assert(df_dl[0].size() == n_dvar,
	 ExcDimensionMismatch(df_dl[0].size(), n_dvar));

  // Set flag to assemble dR/du (Done in solve now)
  this->notify (sensitivity_analysis); // this->assemble(vectors);

  // Assemble dR/dl
  std::vector<unsigned int> block_sizes(this->block_info.local.size());
  for (unsigned int i=0; i<block_sizes.size(); ++i)
    block_sizes[i] = this->block_info.local.block_size(i);
  std::vector<AppFrame::FEVector >dR_dl(n_dvar, AppFrame::FEVector (this->block_info.global));
  this->dresidual_dlambda(dR_dl,
			  vectors);
  // Solve system to obtain -du_dl by solving -dR/du*(-du/dl) = -dR_dl
  std::vector<AppFrame::FEVector >du_dl(n_dvar, AppFrame::FEVector (this->block_info.global));
  for (unsigned int i=0; i<n_dvar; ++i)
    {
      AppFrame::FEVectors aux;
      aux.add_vector(dR_dl[i],"Newton residual");
      unsigned int ind = vectors.find_vector("Solution");
      aux.add_vector(vectors.vector(ind), "Newton iterate");
      this->solve(du_dl[i],
		  aux);//dR_dl[i]);
    }

  // Obtain df_dl
  std::vector<std::vector<double> > pdf_pdl(this->n_resp, std::vector<double> (n_dvar));
  OptimizationBlockMatrixApplication<dim>::dresponses_dl(pdf_pdl,
							 vectors);//sol);
  //print_dresponses_dl(pdf_pdl);

  std::vector<AppFrame::FEVector > df_du(this->n_resp, AppFrame::FEVector (this->block_info.global));
  OptimizationBlockMatrixApplication<dim>::dresponses_du(df_du,
							 vectors);//sol);
  //print_dresponses_du(df_du);

  for (unsigned int i=0; i < df_dl.size(); ++i)
    for (unsigned int j=0; j<df_dl[0].size(); ++j)
      {
	df_dl[i][j] = pdf_pdl[i][j] - df_du[i]*du_dl[j]; //Note the negative sign is due to solving -dR/du*(-du/dl) = -dR_dl
      }

  // Write gradents:
  for (unsigned int i=0; i<n_resp; ++i)
    for (unsigned int j=0; j<n_dvar; ++j)
      std::cout<<"Df"<<i<<"/Dl"<<j<<" is equal to "<<df_dl[i][j]<<std::endl;

}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::solve_adjoint (std::vector<std::vector<double> >& /*df_dl*/,
							const AppFrame::FEVector& /*sol*/)
{

  //For the adjoint I need the transpose of the system matrix... => Guido added a transponse matrix to LAC

}

//---------------------------------------------------------------------------
template <int dim>
unsigned int
OptimizationBlockMatrixApplication<dim>::get_n_resp() const
{
  return n_resp;
}

//---------------------------------------------------------------------------
template <int dim>
unsigned int
OptimizationBlockMatrixApplication<dim>::get_n_dvar() const
{
  return n_dvar;
}

//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::set_optimization_parameters(unsigned int &n_dvar, unsigned int &n_resp,
						   std::vector<std::string> &name_design_var, std::vector<std::string> &name_responses)
{
	this->n_dvar = n_dvar;
	this->n_resp = n_resp;
	this->name_design_var = name_design_var;
	set_output_variables(name_responses);
	optimization = true;
}


//---------------------------------------------------------------------------
template <int dim>
void
OptimizationBlockMatrixApplication<dim>::dfunction(std::vector<AppFrame::FEVector >& dst,
						   const AppFrame::FEVectors& src,
						   bool dfunctional_du,
						   bool dresidual_dlambda)
{
  /*
  AppFrame::FEVectors dummy;

  const Quadrature<dim>& cell_quadrature = this->quadrature_residual_cell;
  const Quadrature<dim-1>& face_quadrature = this->quadrature_residual_face;

  // Loop over all objective function and constraints (functionals)
  for (unsigned int f = 0; f < dst.size(); ++f)
    dst[f].reinit(this->block_info.global);

				   // Generate FEValues object for all
				   // base elements.
  std::vector<FEValues<dim>*> fe(this->element->n_base_elements());
  std::vector<FEFaceValues<dim>*> int_fe(this->element->n_base_elements());
  std::vector<FESubfaceValues<dim>*> int_fe_sub(this->element->n_base_elements());
  std::vector<FEFaceValues<dim>*> ext_fe(this->element->n_base_elements());
				   // and initialize them
  for (unsigned int i=0;i<fe.size();++i)
    {
      fe[i] = new FEValues<dim> (
	*this->mapping,
	this->element->base_element(i),
	cell_quadrature,
	UpdateFlags(update_q_points
		    | update_values
		    | update_gradients
		    | update_JxW_values));

      int_fe[i] = new FEFaceValues<dim> (
	*this->mapping,
	this->element->base_element(i),
	face_quadrature,
	UpdateFlags(update_q_points
		    | update_values
		    | update_gradients
		    | update_normal_vectors
		    | update_JxW_values));
      int_fe_sub[i] = new FESubfaceValues<dim> (
	*this->mapping,
	this->element->base_element(i),
	face_quadrature,
	UpdateFlags(update_q_points
		    | update_values
		    | update_gradients
		    | update_normal_vectors
		    | update_JxW_values));
      ext_fe[i] = new FEFaceValues<dim> (
	*this->mapping,
	this->element->base_element(i),
	face_quadrature,
	UpdateFlags(update_values
		    | update_gradients));
    }

  const unsigned int n_dofs = this->element->dofs_per_cell;
  const unsigned int n_blocks = this->element->n_blocks();
  const unsigned int n_components = this->element->n_components();

				   // Mapping from local to global
				   // indices
  std::vector<unsigned int> indices(n_dofs);
  std::vector<unsigned int> indices_org(n_dofs);
  std::vector<unsigned int> neighbor_indices(n_dofs);
  std::vector<unsigned int> neighbor_indices_org(n_dofs);

                                   // Objects for cell and face data
				   // handed down to the local
				   // routines.
  typename DoFApplication<dim>::CellInfo cell_info(src, fe, indices);

  // Initialize local data vectors
  cell_info.values.resize(src.n_vectors(), std::vector<std::vector<double> >(
			  n_components, std::vector<double>(
				cell_quadrature.size())));

  cell_info.derivatives.resize(src.n_vectors(), std::vector<std::vector<Tensor<1,dim> > >(
				 n_components, std::vector<Tensor<1,dim> >(
				   cell_quadrature.size())));

				   // Result vectors for the cell wise
				   // functions
  std::vector<AppFrame::FEVector > intvec (dst.size(), AppFrame::FEVector (n_blocks));
  std::vector<AppFrame::FEVector > extvec (dst.size(), AppFrame::FEVector (n_blocks));

				   // Cell wise source vector
				   // functions
  std::vector<std::vector<double> > intsrc;
  std::vector<std::vector<double> > extsrc;

  for (unsigned int f = 0; f < dst.size(); ++f)
    {
      for (unsigned int i=0;i<intvec[f].n_blocks();++i)
	{
	  const unsigned int s = this->element->base_element(this->element->block_to_base_index(i).first).dofs_per_cell;
	  intvec[f].block(i).reinit(s);
	  extvec[f].block(i).reinit(s);
	}
      intvec[f].collect_sizes();
      extvec[f].collect_sizes();
    }

  this->tr->clear_user_flags ();

  typename DoFHandler<dim>::active_cell_iterator c;
  for (c = this->dof->begin_active();
       c != this->dof->end();
       ++c)
    {
      // Initialize local structures
      cell_info.reinit(c);
      c->get_dof_indices(indices_org);
      for (unsigned int i=0;i<indices.size();++i)
	indices[this->block_info.local_renumbering[i]] = indices_org[i];

      for (unsigned int f = 0; f < dst.size(); ++f)
	{
	  intvec[f] = 0.;
	  extvec[f] = 0.;
	}

      for (unsigned int i=0;i<fe.size();++i)
	fe[i]->reinit((typename Triangulation<dim>::cell_iterator) c);

      // Fill local data vectors
      this->fill_local_data(cell_info, cell_info.values, src, true);
      this->fill_local_data(cell_info, cell_info.derivatives, src, true);
      // Choose the matrix we want to assemble. I put this if because this way I don't need to
      // code the same thing twice.
      if (dfunctional_du == true)
	cell_dresponses_du(intvec, cell_info,
			   intsrc);
      else if (dresidual_dlambda == true)
	cell_dresidual_dlambda(intvec, cell_info,
			       intsrc);
      else
	{
	  deallog<<"Nothing to be done in optimization_block_matrix_application.cc"<<std::endl;
	  abort();
	  }
      // Insted of:
      for (unsigned int f = 0; f < dst.size(); ++f)
	for (unsigned int i=0;i<n_dofs;++i)
	  dst[f](indices[i]) += intvec[f](i);
      // Use:
      // Can't because intvec is a BlockVector and this routine needs a Vector
      //hanging_node_constraints.distribute_local_to_global(intvec, indices, dst);
    } // cells

  //////////////////////////////////////////////////////////////////////
  // Condense rhs
  //////////////////////////////////////////////////////////////////////
  for (unsigned int f = 0; f < dst.size(); ++f)
    this->hanging_node_constraints.condense(dst[f]);

  //////////////////////////////////////////////////////////////////////
  // Apply Dirichlet boundary conditions
  //////////////////////////////////////////////////////////////////////
  //for (unsigned int f = 0; f < dst.size(); ++f)
  //  this->filtered_matrix.apply_constraints(dst[f],
  //					    false);
  */
}
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
template class AppFrame::OptimizationBlockMatrixApplication<deal_II_dimension>;