/*
 * agglomerate_base.cc
 *
 *  Created on: Mar 25, 2013
 *      Author: wardlawp
 */

#include "agglomerate_base.h"


namespace NAME = FuelCellShop::MicroScale;

void

NAME::AgglomerateBase::setUpLoadings(){

	if (loadingWeigths.size() != 0){

		for(int i = 0; i <= loadingWeigths.size(); i++)
			loadingRadii.push_back( (1.0/loadingWeigths.size()) * i );

		double weightingSum =0;

		for(int i = 0; i < loadingWeigths.size(); i++)
			weightingSum = weightingSum  + loadingWeigths[i]*((loadingRadii[i+1]*loadingRadii[i+1]) - (loadingRadii[i]*loadingRadii[i]));

		for(int i = 0; i < loadingWeigths.size(); i++)
			actualLoadings.push_back(double(AV/weightingSum)*loadingWeigths[i]);

	}
}


double
NAME::AgglomerateBase::getAV(double location){


	if(actualLoadings.size() != loadingWeigths.size())
		setUpLoadings();

	if (loadingWeigths.size() == 0)
		return AV;


	for(int i = 0; i < actualLoadings.size(); i++)
		if(location >= loadingRadii[i] && location < loadingRadii[i+1]){
			return actualLoadings[i];
		}

	if(location == 1)
		return actualLoadings.back();

	return -1;


}

void
NAME::AgglomerateBase::deep_copy_pointers(){
    bool fail = false;
    std::string fail_type;
    if(typeid(FuelCellShop::Material::Platinum).name() == typeid(*this->catalyst).name()){
        catalyst = new FuelCellShop::Material::Platinum
                            (*dynamic_cast<FuelCellShop::Material::Platinum*>(this->catalyst));
    }
    else{
        fail = true;
        fail_type = typeid(*this->catalyst).name();
    }


    if(!fail and typeid(FuelCellShop::Material::Nafion).name() == typeid(*this->electrolyte).name()){
        electrolyte = new FuelCellShop::Material::Nafion
                            (*dynamic_cast<FuelCellShop::Material::Nafion*>(this->electrolyte));
    }
    else{
        fail = true;
        fail_type = typeid(*this->electrolyte).name();
    }

    if(!fail and typeid(FuelCellShop::Kinetics::TafelKinetics).name() == typeid(*this->kinetics).name()){
        kinetics = new FuelCellShop::Kinetics::TafelKinetics
                            (*dynamic_cast<FuelCellShop::Kinetics::TafelKinetics*>(this->kinetics));
    }
    else{
        fail = true;
        fail_type = typeid(*this->electrolyte).name();
    }

    if (fail){
        std::cerr << "Agglomerate: casting of object type " + fail_type + " not yet implemented, please run in serial model" << std::endl;
        exit(1);
    }

}

void
NAME::AgglomerateBase::interpolate_initial_data(double z[], double &x, double left_pos, double right_pos, std::vector<double> left_data, std::vector<double> right_data)
{
    //start at i=1 to ignore the first element
    for(int i = 1; i<left_data.size(); i++)
        z[i-1] = left_data.at(i) + ((right_data.at(i) - left_data.at(i))*(x - left_pos))/(right_pos - left_pos);

}

bool
NAME::AgglomerateBase::use_initial_data(double z[], double &x){
    bool answer = false;

    if(initial_data.size() != 0){


        //Start at centre, centre values are 0
        double left = 0;
        std::vector<double> left_data;
        for(int j =0; j<initial_data[0].size(); j++)
            left_data.push_back(0);

        double right;
        for(int i = 0; i <initial_data.size(); i++){

            right = initial_data[i][0];
            if( (left<= x) && (x <=  right)){
                interpolate_initial_data(z,x,left,right, left_data, initial_data[i]);
                answer = true;
                break;
            }

            left = right;
            left_data = initial_data[i];
        }

        //If we went past the right most point and haven't yet interpolated
        if (!answer and x > right){
            for(int k = 0; k < initial_data[0].size() -1; k++)
                z[k] = initial_data[initial_data.size()-1][k+1];

            answer = true;
        }

    }

    return answer;

}


void
NAME::AgglomerateBase::set_solution(const std::map<VariableNames,SolutionVariable>& sols,const VariableNames& react, const int& solution_index){


    typedef std::map<VariableNames,SolutionVariable>::const_iterator map_itr;

    //Check for key variables (x_O2/x_H2, phi_M, and phi_S)
    AssertThrow((react == oxygen_molar_fraction) or (react == hydrogen_molar_fraction) ,
            ExcMessage("Analytical ionomer agglomerate cannot solve for this type of reactant."));

    AssertThrow(sols.find(protonic_electrical_potential) != sols.end(),
                ExcMessage("Solution is missing protonic potential!"));

    AssertThrow((sols.find(electronic_electrical_potential) != sols.end()),
                    ExcMessage("Solution is missing solid potential!"));

    this->sol_index = solution_index;
    this->solutions = sols;

    this->sol_names.clear();
    this->sol_names.push_back(react);
    this->sol_names.push_back(protonic_electrical_potential);
    this->sol_names.push_back(electronic_electrical_potential);

    this->reactant = react;

}
