// ----------------------------------------------------------------------------
//
// FCST: Fuel Cell Simulation Toolbox
//
// Copyright (C) 2006-2013 by Energy Systems Design Laboratory, University of Alberta
//
// This software is distributed under the MIT License
// For more information, see the README file in /doc/LICENSE
//
// - Class: base_layer.cc
// - Description: This is a base class for all available FCST layers
// - Developers: Marc Secanell Gallart,    University of Alberta
//               Madhur Bhaiya, University of Alberta
// - $Id: base_layer.cc 1354 2013-08-17 00:01:22Z secanell $
//
// ----------------------------------------------------------------------------

#include "base_layer.h"

namespace NAME = FuelCellShop::Layer;

//---------------------------------------------------------------------------
template <int dim>
NAME::BaseLayer<dim>::BaseLayer(const std::string& name)
  : name(name)
{
  //deallog<<"Creating layer " << name<<std::endl;
}

//---------------------------------------------------------------------------
template <int dim>
NAME::BaseLayer<dim>::~BaseLayer()
{}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::BaseLayer<dim>::initialize (ParameterHandler &param)
{
  param.enter_subsection("Fuel cell data");
  {
    param.enter_subsection(this->name);
    {
      material_id = param.get_integer("Material id");
    }
    param.leave_subsection();
  }
  param.leave_subsection();
}

//---------------------------------------------------------------------------
template <int dim>
bool
NAME::BaseLayer<dim>::belongs_to_material(const char material)
{
  return material == material_id;
}

//---------------------------------------------------------------------------
template <int dim>
const std::string&
NAME::BaseLayer<dim>::name_material()
{
  return name;
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::BaseLayer<dim>::print_layer_properties() const
{
	  const std::type_info& info = typeid(*this);
	  deallog << "Pure function " << __FUNCTION__
		  << " called in Class "
		  << info.name() << std::endl;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Explicit instantiations.
template class NAME::BaseLayer<deal_II_dimension>;