//---------------------------------------------------------------------------
//
//    FCST: Fuel Cell Simulation Toolbox
//
//    Copyright (C) 2012, 2013 by Energy Systems Design Laboratory, University of Alberta
//
//    This software is distributed under the MIT License.
//    For more information, see the README file in /doc/LICENSE
//
//    - Class: dummy_GDL.cc
//    - Description: Implementation of a GDL class that setup us all properties from file
//    - Developers: M. Secanell
//    - Id: $Id: dummy_GDL.cc 1460 2013-08-30 22:28:49Z madhur $
//
//---------------------------------------------------------------------------

#include <dummy_GDL.h>

namespace NAME = FuelCellShop::Layer; 

template <int dim>
const std::string NAME::DummyGDL<dim>::concrete_name ("DummyGDL");

template <int dim>
NAME::DummyGDL<dim> const* NAME::DummyGDL<dim>::PROTOTYPE = new NAME::DummyGDL<dim>();


//---------------------------------------------------------------------------
template <int dim>
NAME::DummyGDL<dim>::DummyGDL()
: FuelCellShop::Layer::GasDiffusionLayer<dim> ()
{
    //deallog<<" Register DummyGDL GDL to FactoryMap"<<std::endl;
    this->get_mapFactory()->insert(std::pair<std::string, FuelCellShop::Layer::GasDiffusionLayer<dim>* >(concrete_name, this));
}
            
            
//---------------------------------------------------------------------------
template <int dim>
NAME::DummyGDL<dim>::DummyGDL(std::string name)
  : NAME::GasDiffusionLayer<dim> (name)
{
  deallog<<" of type DummyGDL"<<std::endl;
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::declare_parameters (const std::string& name, 
                                         ParameterHandler &param) const
{
    
    FuelCellShop::Layer::GasDiffusionLayer<dim>::declare_parameters(name, param);
    
    param.enter_subsection("Fuel cell data");
    {
        param.enter_subsection(name);
        {
            param.enter_subsection(concrete_name); //-- Transport for the anisotropic case:
            {
                
                // ISOTROPIC PROPERTIES:
                param.declare_entry ("Oxygen diffusion coefficient, [cm^2/s]",
                                     "0.2741", //1atm, 353K
                                     Patterns::Double(),
                                     "Oxygen diffusion coefficient given by experiment");
                param.declare_entry ("Water vapour diffusion coefficient, [cm^2/s]",
                                     "0.29646", 
                                     Patterns::Double(),
                                     "Water vapour diffusion coefficient given by experiment");
                param.declare_entry ("Electrical conductivity, [S/cm]",
                                     "100", // [S/cm]
                                     Patterns::Double(),
                                     "Effective cond. if given is used, otherwise conducitivity of the raw material. Units [S/cm]");
                // ANISOTROPIC PROPERTIES IF NEEDED:    
                param.declare_entry ("Anisotropic transport",
                                     "false", // [S/cm]
                                     Patterns::Bool(),
                                     "Boolean variable. Set to true if we want to account for anisotropy of the GDL");
                //--- XX
                param.declare_entry ("Oxygen diffusion coefficient X, [cm^2/s]",
                                     "0.2741", //1atm, 353K
                                     Patterns::Double(),
                                     "Oxygen diffusion coefficient given by experiment");
                param.declare_entry ("Water vapour diffusion coefficient X, [cm^2/s]",
                                     "0.29646", 
                                     Patterns::Double(),
                                     "Water vapour diffusion coefficient given by experiment");
                param.declare_entry ("Electrical conductivity X, [S/cm]",
                                     "100", // [S/cm]
                                     Patterns::Double(),
                                     "Component X of the electrical conductivity tensor. Units [S/cm]");
                // YY
                param.declare_entry ("Oxygen diffusion coefficient Y, [cm^2/s]",
                                     "0.2741", //1atm, 353K
                                     Patterns::Double(),
                                     "Oxygen diffusion coefficient given by experiment");
                param.declare_entry ("Water vapour diffusion coefficient Y, [cm^2/s]",
                                     "0.29646", 
                                     Patterns::Double(),
                                     "Water vapour diffusion coefficient given by experiment");
                param.declare_entry ("Electrical conductivity Y, [S/cm]",
                                     "100", // [S/cm]
                                     Patterns::Double(),
                                     "Component Y of the electrical conductivity tensor. Units [S/cm]");
                // ZZ
                param.declare_entry ("Oxygen diffusion coefficient Z, [cm^2/s]",
                                     "0.2741", //1atm, 353K
                                     Patterns::Double(),
                                     "Oxygen diffusion coefficient given by experiment");
                param.declare_entry ("Water vapour diffusion coefficient Z, [cm^2/s]",
                                     "0.29646", 
                                     Patterns::Double(),
                                     "Water vapour diffusion coefficient given by experiment");
                param.declare_entry ("Electrical conductivity Z, [S/cm]",
                                     "100", 
                                     Patterns::Double(),
                                     "Component Z of the electrical conductivity tensor. Units [S/cm]");
                
            }
            param.leave_subsection();
            
        }
        param.leave_subsection();        
    }
    param.leave_subsection();
    
}
//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::initialize (ParameterHandler &param)
{
    
    NAME::GasDiffusionLayer<dim>::initialize(param);
    
    param.enter_subsection("Fuel cell data"); 
    {
        param.enter_subsection(this->name); 
        { 
            param.enter_subsection(concrete_name); 
            { 
                // Anisotropy
                anisotropy = param.get_bool ("Anisotropic transport");
                
                D_O2.resize(dim);
                D_wv.resize(dim);
                sigma_e.resize(dim);
                
                if (anisotropy == true)
                {
                    switch (dim)
                    {
                    // X
                    case 1:
                        D_O2[0] = param.get_double("Oxygen diffusion coefficient X, [cm^2/s]");
                        D_wv[0] = param.get_double("Water vapour diffusion coefficient X, [cm^2/s]");
                        sigma_e[0] = param.get_double("Electrical conductivity X, [S/cm]");                     
                        break;

                    case 2:
                        D_O2[0] = param.get_double("Oxygen diffusion coefficient X, [cm^2/s]");
                        D_wv[0] = param.get_double("Water vapour diffusion coefficient X, [cm^2/s]");
                        sigma_e[0] = param.get_double("Electrical conductivity X, [S/cm]");   
                        // Y
                        D_O2[1] = param.get_double("Oxygen diffusion coefficient Y, [cm^2/s]");
                        D_wv[1] = param.get_double("Water vapour diffusion coefficient Y, [cm^2/s]");
                        sigma_e[1] = param.get_double("Electrical conductivity Y, [S/cm]");
                        break;
                        
                    case 3:
                        D_O2[0] = param.get_double("Oxygen diffusion coefficient X, [cm^2/s]");
                        D_wv[0] = param.get_double("Water vapour diffusion coefficient X, [cm^2/s]");
                        sigma_e[0] = param.get_double("Electrical conductivity X, [S/cm]");   
                        // Y
                        D_O2[1] = param.get_double("Oxygen diffusion coefficient Y, [cm^2/s]");
                        D_wv[1] = param.get_double("Water vapour diffusion coefficient Y, [cm^2/s]");
                        sigma_e[1] = param.get_double("Electrical conductivity Y, [S/cm]");
                        // Z
                        D_O2[2] = param.get_double("Oxygen diffusion coefficient Z, [cm^2/s]");	  
                        D_wv[2] = param.get_double("Water vapour diffusion coefficient Z, [cm^2/s]");
                        sigma_e[2] = param.get_double("Electrical conductivity Z, [S/cm]");
                        break;
                    }
                }
                else
                {
                    for (unsigned int i =0; i < D_O2.size(); i++)
                    {
                        D_O2[i] = param.get_double("Oxygen diffusion coefficient, [cm^2/s]");
                        D_wv[i] = param.get_double("Water vapour diffusion coefficient, [cm^2/s]");
                        sigma_e[i] = param.get_double("Electrical conductivity, [S/cm]");
                    }
                }
            }
            param.leave_subsection(); 
        }
        param.leave_subsection(); 
    }
    param.leave_subsection();
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::set_parameters (const std::vector<std::string>& name_dvar,
                                     const std::vector<double>& value_dvar,
                                     const std::string& gdl_section_name, 
                                     ParameterHandler &param)
{
    param.enter_subsection("Fuel cell data");
    {
        param.enter_subsection(gdl_section_name);
        {
            param.enter_subsection(concrete_name); 
            { 
                for (unsigned int i=0; i<name_dvar.size(); ++i)
                {
                    if (name_dvar[i] == "D_O2_CGDL")
                    {
                        if(this->name == "Cathode gas diffusion layer")
                            param.set("Oxygen diffusion coefficient, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_H2O_CGDL")
                    {
                        if(this->name == "Cathode gas diffusion layer")
                            param.set("Water vapour diffusion coefficient, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_H2O_AGDL")
                    {
                        if(this->name == "Anode gas diffusion layer")
                            param.set("Water vapour diffusion coefficient, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_O2_CGDL_X")
                    {
                        if(this->name == "Cathode gas diffusion layer")
                            param.set("Oxygen diffusion coefficient X, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_H2O_CGDL_X")
                    {
                        if(this->name == "Cathode gas diffusion layer")
                            param.set("Water vapour diffusion coefficient X, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_H2O_AGDL_X")
                    {
                        if(this->name == "Anode gas diffusion layer")
                            param.set("Water vapour diffusion coefficient X, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_O2_CGDL_Y")
                    {
                        if(this->name == "Cathode gas diffusion layer")
                            param.set("Oxygen diffusion coefficient Y, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_H2O_CGDL_Y")
                    {
                        if(this->name == "Cathode gas diffusion layer")
                            param.set("Water vapour diffusion coefficient Y, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "D_H2O_AGDL_Y")
                    {
                        if(this->name == "Anode gas diffusion layer")
                            param.set("Water vapour diffusion coefficient Y, [cm^2/s]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "sigma_e_GDL_X")
                    {
                        param.set("Electrical conductivity X, [S/cm]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "sigma_e_GDL_Y")
                    {
                        param.set("Electrical conductivity Y, [S/cm]", value_dvar[i]);
                    }
                    else if (name_dvar[i] == "sigma_e_GDL")
                    {
                        param.set("Electrical conductivity, [S/cm]", value_dvar[i]);
                    }
                }
            }
            param.leave_subsection();
        }
        param.leave_subsection();
    }
    param.leave_subsection();
}
//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::effective_gas_diffusivity(Table< 2, double> & prop_eff) const
{
    prop_eff.reinit(this->gases.size(),this->gases.size());
    
    unsigned int dimension = 0;

    for (unsigned int i = 0; i<this->gases.size(); i++)
    {
        std::string solute_name(this->gases[i]->get_name());

        for (unsigned int j = i+1; j<this->gases.size(); j++)
        {  
            std::string solvent_name(this->gases[j]->get_name());
            
            if (solute_name.compare("oxygen") == 0)
            {
                
                if (solvent_name.compare("nitrogen") == 0)
                {
                    prop_eff(i,j) = Units::convert(D_O2[dimension], Units::UNIT2, Units::C_UNIT2);
                    prop_eff(j,i) = prop_eff(i,j);
                    break;
                }
                
            }
            else if (solute_name.compare("water") == 0)
            {
                
                if (solvent_name.compare("nitrogen") == 0)
                {
                    prop_eff(i,j) = Units::convert(D_wv[dimension], Units::UNIT2, Units::C_UNIT2);
                    prop_eff(j,i) = prop_eff(i,j);
                    break;
                }
            }
        }
    }
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::effective_gas_diffusivity(Table< 2, Tensor< 2, dim > > &prop_eff) const
{  
    
    prop_eff.reinit(this->gases.size(),this->gases.size());
    
    for (unsigned int i = 0; i<this->gases.size(); i++)
    {
        std::string name_i;
        name_i = this->gases[i]->get_name();
        
        for (unsigned int j = i+1; j<this->gases.size(); j++)
        {  
            std::string name_j;
            name_j = this->gases[j]->get_name();
            
            if (name_i.compare("oxygen") == 0)
            {
                
                if (name_j.compare("nitrogen") == 0)
                {
                    for (unsigned int d = 0; d<dim; d++)
                    {
                        prop_eff(i,j)[d][d] = Units::convert(D_O2[d], Units::UNIT2, Units::C_UNIT2);
                        prop_eff(j,i)[d][d] = prop_eff(i,j)[d][d];                       
                    }
                }
                else
                {
                    deallog << "Species "<< name_j.c_str() << " diffusivity requested in"
                    << "DummyGDL<dim>::effective_gas_diffusivity"
                    <<"not implemented"<< std::endl;
                    exit(1);
                } 
                
            }
            else if (name_i.compare("water") == 0)
            {
                
                if (name_j.compare("nitrogen") == 0)
                {
                    for (unsigned int d = 0; d<dim; d++)
                    {
                        prop_eff(i,j)[d][d] = Units::convert(D_wv[d], Units::UNIT2, Units::C_UNIT2);
                        prop_eff(j,i)[d][d] = prop_eff(i,j)[d][d];
                        break;
                    }
                }
                else
                {
                    deallog << "Species "<< name_j.c_str() << " diffusivity requested in"
                    << "DummyGDL<dim>::effective_gas_diffusivity"
                    <<"not implemented"<< std::endl;
                    exit(1);
                } 
            }
            else
            {
                deallog << "Species "<< name_i.c_str() << " diffusivity requested in"
                << "DummyGDL<dim>::effective_gas_diffusivity"
                <<"not implemented"<< std::endl;
                exit(1);
            } 
        }
    }
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::effective_electron_conductivity(double& prop_eff) const
{
  if (anisotropy == false)
  {
    prop_eff = sigma_e[0];
  }
  else
  {
    deallog << "The member function " << __FUNCTION__
		  << " called in Class DummyGDL"
		  << "can only be used for isotropic materials. Set anisotropy to false"
		  <<std::endl;
  }
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::DummyGDL<dim>::effective_electron_conductivity(Tensor<2,dim>& prop_eff) const
{
     for (unsigned int i=0; i<dim; i++)
       prop_eff[i][i] = this->sigma_e[i]; // Include in declare paramters and initialize.
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Explicit instantiations. 
template class NAME::DummyGDL<deal_II_dimension>;