//---------------------------------------------------------------------------
//
//    FCST: Fuel Cell Simulation Toolbox
//
//    Copyright (C) 2006-13 by Energy Systems Design Laboratory, University of Alberta
//
//    This software is distributed under the MIT License.
//    For more information, see the README file in /doc/LICENSE
//
//    - Class: nafion_membrane.cc
//    - Description: Class representing Nafion membrane layer class - returning effective transport properties
//    - Developers: Madhur Bhaiya (2012-13)
//    - Id: $Id: nafion_membrane.cc 1460 2013-08-30 22:28:49Z madhur $ 
//
//---------------------------------------------------------------------------

#include "layers/nafion_membrane.h"

namespace NAME = FuelCellShop::Layer;

template <int dim>
const std::string NAME::NafionMembrane<dim>::concrete_name ("NafionMembrane");

template <int dim>
NAME::NafionMembrane<dim> const* NAME::NafionMembrane<dim>::PROTOTYPE = new NAME::NafionMembrane<dim>();

//---------------------------------------------------------------------------
template <int dim>
NAME::NafionMembrane<dim>::NafionMembrane()
  : NAME::MembraneLayer<dim>()
{
    //deallog<<" Register NafionMembrane to FactoryMap"<<std::endl;
    this->get_mapFactory()->insert(std::pair<std::string, NafionMembrane<dim>* > (this->concrete_name, this) );     
}

//---------------------------------------------------------------------------
template <int dim>
NAME::NafionMembrane<dim>::NafionMembrane(std::string name)
  : NAME::MembraneLayer<dim>(name)
{
    deallog<<" Created a NafionMembrane"<<std::endl;
}

//---------------------------------------------------------------------------
template <int dim>
NAME::NafionMembrane<dim>::~NafionMembrane()
{}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::declare_parameters(const std::string& pem_section_name,
                                              ParameterHandler &param) const
{
    NAME::MembraneLayer<dim>::declare_parameters(pem_section_name, param);
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::NafionMembrane<dim>::set_parameters (const std::vector<std::string>& name_dvar,
                                           const std::vector<double>& value_dvar,
                                           const std::string& pem_section_name, 
                                           ParameterHandler &param) const
{
    
    NAME::MembraneLayer<dim>::set_parameters(name_dvar, value_dvar, pem_section_name, param);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::initialize (ParameterHandler &param)
{
    FuelCellShop::Layer::MembraneLayer<dim>::initialize(param);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::effective_proton_conductivity(double& sigma_eff) const
{
    this->electrolyte->proton_conductivity(sigma_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::effective_proton_conductivity(std::vector<double>& sigma_eff) const
{
    this->electrolyte->proton_conductivity(sigma_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::derivative_effective_proton_conductivity(std::map< VariableNames, std::vector<double> >& dSigma_eff) const
{
    Assert(this->derivative_flags.size()!=0, ExcMessage("set_derivative_flags has not been probably called before NafionMembrane::derivative_effective_proton_conductivity."));
    this->electrolyte->proton_conductivity_derivative(dSigma_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::effective_water_diffusivity(double& D_w_eff) const
{
    this->electrolyte->water_diffusivity(D_w_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::effective_water_diffusivity(std::vector<double>& D_w_eff) const
{
    this->electrolyte->water_diffusivity(D_w_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::derivative_effective_water_diffusivity(std::map< VariableNames, std::vector<double> >& dD_w_eff) const
{
    Assert(this->derivative_flags.size()!=0, ExcMessage("set_derivative_flags has not been probably called before NafionMembrane::derivative_effective_water_diffusivity."));
    this->electrolyte->water_diffusivity_derivative(dD_w_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::effective_oxygen_diffusivity(double& D_O_eff) const
{
    this->electrolyte->oxygen_diffusivity(D_O_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::effective_oxygen_diffusivity(std::vector<double>& D_O_eff) const
{
    this->electrolyte->oxygen_diffusivity(D_O_eff);
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::NafionMembrane<dim>::derivative_effective_oxygen_diffusivity(std::map< VariableNames, std::vector<double> >& dD_O_eff) const
{
    Assert(this->derivative_flags.size()!=0, ExcMessage("set_derivative_flags has not been probably called before NafionMembrane::derivative_effective_oxygen_diffusivity."));
    this->electrolyte->oxygen_diffusivity_derivative(dD_O_eff);
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Explicit instantiations. 
template class NAME::NafionMembrane<deal_II_dimension>;
