//---------------------------------------------------------------------------
//    $Id: solid_layer.cc 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2009 by Marc Secanell, University of Alberta
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <solid_layer.h>

namespace NAME = FuelCellShop::Layer;

//---------------------------------------------------------------------------
template <int dim>
NAME::SolidLayer<dim>::SolidLayer(std::string name)
  : NAME::BaseLayer<dim> (name)
{}

//---------------------------------------------------------------------------
template <int dim>
NAME::SolidLayer<dim>::~SolidLayer()
{}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::SolidLayer<dim>::declare_parameters (ParameterHandler &param) const
{
  NAME::BaseLayer<dim>::declare_parameters(param);

  param.enter_subsection("Fuel cell data");
  {
    param.enter_subsection(this->name);
  {
    //-- Geometry (here or in the mesh)
   param.declare_entry ("Boundary id of the solid-ionomer interface",
			 "0",
			 Patterns::Integer(),
			 "ID used to read mesh");
   param.declare_entry ("Boundary id of the solid-pore interface",
			"1",
			Patterns::Integer(),
			"ID used to read mesh");
   
   }
  param.leave_subsection();
}
  param.leave_subsection();
}

//---------------------------------------------------------------------------
template <int dim>
void
NAME::SolidLayer<dim>::set_parameters (const std::vector<std::string>& /*name_dvar*/,
						       const std::vector<double>& /*value_dvar*/,
						       ParameterHandler & /*param*/)
{}


//---------------------------------------------------------------------------
template <int dim>
void
NAME::SolidLayer<dim>::initialize (ParameterHandler &param)
{

  NAME::BaseLayer<dim>::initialize(param);

  param.enter_subsection("Fuel cell data");
  {
    param.enter_subsection(this->name);
    {
      boundary_ids.push_back(param.get_integer("Boundary id of the solid-ionomer interface"));
      boundary_ids.push_back(param.get_integer("Boundary id of the solid-pore interface"));
     
    }
    param.leave_subsection();
  }
  param.leave_subsection();
  /*
  deallog<<"Porosity: "<<porosity<<std::endl;
  for (unsigned int i =0; i < porosity_th.size(); i++)
    {
      deallog<<"porosity_th["<<i<<"]: "<<porosity_th[i]<<std::endl;
      deallog<<"porosity_mu["<<i<<"]: "<<porosity_mu[i]<<std::endl;
      deallog<<" porosity_gamma["<<i<<"]: "<< porosity_gamma[i]<<std::endl;
    }
  abort();
  */
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::SolidLayer<dim>::set_solid_and_compute (FuelCellShop::Material::FiberBase* fiber,
					     double temperature)
{
  this->solid = fiber;
  this->temperature = temperature;
  this->electrical_conductivity = this->solid->get_electrical_conductivity(temperature);
 
};

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::SolidLayer<dim>::effective_electron_conductivity(double& sigma_eff ) const
{
 sigma_eff = this->electrical_conductivity; 
}

//---------------------------------------------------------------------------
template <int dim>
void 
NAME::SolidLayer<dim>::test_class()
{
   NAME:SolidLayer<dim> tina("name of subsection in parameter file");
   FuelCellShop::Material::CarbonFiber cf;
   tina.set_solid_and_compute(&cf,298);
   double val;
   tina.effective_electron_conductivity(val);
   deallog<<"The computed conductivity is: "<<val<<std::endl;
} 
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Explicit instantiations.
template class NAME::SolidLayer<deal_II_dimension>;
