//---------------------------------------------------------------------------
//    $Id: MixtureDynamicViscosity.cc 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2010 by Ali MalekpourKoupaei
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------
#include <MixtureDynamicViscosity.h>
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
FuelCellShop::Mixture::MixtureDynamicViscosity::MixtureDynamicViscosity()
//---------------------------------------------------------------------------
{
  species1 = __null;
  species2 = __null;
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::MixtureDynamicViscosity::MixtureDynamicViscosity(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2)
//---------------------------------------------------------------------------
{
  this->species1 = &species1;
  this->species2 = &species2;
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::MixtureDynamicViscosity::~MixtureDynamicViscosity()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::MixtureDynamicViscosity::set_T_n_xi(const double& T, const std::vector<double>& xi)
//---------------------------------------------------------------------------
{
  this->T = T;
  this->xi = xi;
}

void 
FuelCellShop::Mixture::MixtureDynamicViscosity::set_species(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2)
//---------------------------------------------------------------------------
{
  this->species1 = &species1;
  this->species2 = &species2;
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::MixtureDynamicViscosity::scan()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::MixtureDynamicViscosity::report()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::MixtureDynamicViscosity::report2file(char* filename)
//---------------------------------------------------------------------------
{
  std::ofstream out(filename);
  out << "Method name: " << get_name() << "\n";
  out << "Method ID: " << get_ID() << std::endl;
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::VaporMixtureDynamicViscosity::VaporMixtureDynamicViscosity()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::VaporMixtureDynamicViscosity::VaporMixtureDynamicViscosity
	(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2)
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::VaporMixtureDynamicViscosity::~VaporMixtureDynamicViscosity ()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::VaporMixtureDynamicViscosity::scan()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::VaporMixtureDynamicViscosity::report()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::VaporMixtureDynamicViscosity::report2file(char* filename)
//---------------------------------------------------------------------------
{
}


//---------------------------------------------------------------------------
//CHAPMAN-ENSKOG METHOD IMPLEMENTATION---------------------------------------
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::ChapmanEnskogViscosityModel()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::ChapmanEnskogViscosityModel(FuelCellShop::Material::PureGas& species1, 		    
      FuelCellShop::Material::PureGas& species2)
//---------------------------------------------------------------------------
{
  this->species1 = &species1;
  this->species2 = &species2;
}

//---------------------------------------------------------------------------
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::~ChapmanEnskogViscosityModel()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
char* FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_name()
//---------------------------------------------------------------------------
{
  return "ChapMan-Enskog mixture Viscosity Model";
}

//---------------------------------------------------------------------------
char* FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_description()
//---------------------------------------------------------------------------
{
  return "";
}

//---------------------------------------------------------------------------
enDynamicViscosityModelID
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_ID()
//---------------------------------------------------------------------------
{
  return vmChapmanEnskog;
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_exi(std::vector< std::vector<double> >& exi, const double& T)
{
  exi.resize(2);
  exi[0].resize(2);
  exi[1].resize(2);

  std::vector<double> MW(exi.size());
  MW[0]=species1->M;
  MW[1]=species2->M;

  std::vector<double> eta_0(exi.size());
  species1->get_viscosity_Chapman_Enskog(eta_0[0], T);
  species2->get_viscosity_Chapman_Enskog(eta_0[1], T);

  for(int i=0; i<2; ++i)
    for(int j=0; j<2; ++j)
      exi[i][j] = pow((1+sqrt(eta_0[i]/eta_0[j])*pow((MW[j]/MW[i]),0.25)),2)/sqrt(8*(1+MW[i]/MW[j]));
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_Mu (double& Mu)
//---------------------------------------------------------------------------
{
  get_Mu (Mu, T, xi);
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_Mu (double& Mu, const double& T, const std::vector<double>& xi)
//---------------------------------------------------------------------------
{
  Mu = 0;
  std::vector< std::vector<double> > exi; 
  get_exi(exi, T);

  std::vector<double> eta_0(exi.size()); 
  species1->get_viscosity_Chapman_Enskog(eta_0[0],T);
  species2->get_viscosity_Chapman_Enskog(eta_0[1],T);

  double xi_exi_ij=0;
  for(int i=0; i<xi.size(); ++i)
  {
    for(int j=0; j<xi.size(); ++j)
      xi_exi_ij += xi[j]*exi[i][j];
    Mu += xi[i]*eta_0[i]/xi_exi_ij;
    xi_exi_ij=0;
  }
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_dMu_dxi (double& dMu, const double& T, const std::vector<double>& xi)
//---------------------------------------------------------------------------
{
  std::vector< std::vector<double> > exi(2); 
  exi[0].resize(2);
  exi[1].resize(2);
  get_exi(exi, T);

  std::vector<double> xi_exi_ij(2);
  for(int i=0; i<xi.size(); ++i)
  {
    xi_exi_ij[i]=0;
    for(int j=0; j<xi.size(); ++j)
      xi_exi_ij[i] += xi[j]*exi[i][j];
  }

  std::vector<double> eta_0(exi.size()); 
  species1->get_viscosity_Chapman_Enskog(eta_0[0],T);
  species2->get_viscosity_Chapman_Enskog(eta_0[1],T);

  dMu = eta_0[0]*(exi[0][1]/pow(xi_exi_ij[0],2)) - eta_0[1]*(exi[1][0]/pow(xi_exi_ij[1],2));
}

//---------------------------------------------------------------------------
void 
FuelCellShop::Mixture::ChapmanEnskogViscosityModel::get_dMu_dT (double& dMu, const double& T, const std::vector<double>& xi)
//---------------------------------------------------------------------------
{
  dMu = 0;
  std::vector< std::vector<double> > exi; 
  get_exi(exi, T);

  std::vector<double> d_eta_0(exi.size()); 
  species1->get_dMu_d_T_Chapman_Enskog(d_eta_0[0],T);
  species2->get_dMu_d_T_Chapman_Enskog(d_eta_0[1],T);

  double xi_exi_ij=0;
  for(int i=0; i<xi.size(); ++i)
  {
    for(int j=0; j<xi.size(); ++j)
      xi_exi_ij += xi[j]*exi[i][j];
    dMu += xi[i]/xi_exi_ij * d_eta_0[i];
    xi_exi_ij=0;
  }
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::ChapmanEnskogViscosityModel::scan()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::ChapmanEnskogViscosityModel::report()
//---------------------------------------------------------------------------
{
}

//---------------------------------------------------------------------------
void FuelCellShop::Mixture::ChapmanEnskogViscosityModel::report2file(char* filename)
//---------------------------------------------------------------------------
{
}
