//---------------------------------------------------------------------------
//    $Id: PureLiquid.cc 1354 2013-08-17 00:01:22Z secanell $
//
//    Copyright (C) 2012 by Madhur Bhaiya
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include "materials/PureLiquid.h"

namespace NAME = FuelCellShop::Material;

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//LIQUID WATER-------------------------------------------------------------------
//---------------------------------------------------------------------------
NAME::LiquidWater::LiquidWater()
{
  M   = 18.015; // g/mol
}
//---------------------------------------------------------------------------
NAME::LiquidWater::~LiquidWater()
{
}
//---------------------------------------------------------------------------
char* 
NAME::LiquidWater::get_name()
{
  return "Liquid Water";
}
//---------------------------------------------------------------------------
char* 
NAME::LiquidWater::get_formula()
{
  return "H2O (liquid)";
}

//---------------------------------------------------------------------------
void 
NAME::LiquidWater::get_viscosity(double& mu,const double& T) 
{
std::cout << "Function not defined yet..." << std::endl;
exit(1);
}

//---------------------------------------------------------------------------
double
NAME::LiquidWater::latentVap_heat(const double& Temp)
{
	double T_c = Temp - 273.15;
	double h_fg = (2500.304 - (2.2521025*T_c) - (0.021465847*(std::pow(T_c,1.5))) + (3.1750136e-4*(std::pow(T_c,2.5))) - (2.8607959e-5*(std::pow(T_c,3.0)))) * 18.015; // J/mol
	return h_fg;
}

//---------------------------------------------------------------------------
double
NAME::LiquidWater::deriv_latentVap_heat(const double& Temp)
{
	double T_c = Temp - 273.15;
	double Dh_fg_DT = ((3.1750136e-4*2.5*(std::pow(T_c,1.5))) - 2.2521025 - (0.021465847*1.5*(std::pow(T_c,0.5))) - (2.8607959e-5*3.0*(std::pow(T_c,2.0)))) * 18.015; //J/mol-(K)
	return Dh_fg_DT;
}

void
NAME::LiquidWater::declare_parameters (ParameterHandler &param) const
{

	param.enter_subsection("Fuel cell data");
	{
		param.enter_subsection("Materials");
		{
			param.enter_subsection("Water");
			{

				param.declare_entry("Oxygen diffusion coefficient [cm^2/s]",
								"9.19e-5",
								Patterns::Double());
				param.declare_entry("Proton diffusion coefficient [cm^2/s]",
											"9.2e-5",
											Patterns::Double());
				param.declare_entry("Relative permittivity",
											"60",
											Patterns::Double());
			}
			param.leave_subsection();
		}
		param.leave_subsection();
	}

	param.leave_subsection();
}

void
NAME::LiquidWater::initialize(ParameterHandler &param)
{


	param.enter_subsection("Fuel cell data");
		{
		param.enter_subsection("Materials");
		{
			param.enter_subsection("Water");
			{
				oxygen_diffusion_coeff = param.get_double("Oxygen diffusion coefficient [cm^2/s]");
				proton_diffusion_coeff = param.get_double("Proton diffusion coefficient [cm^2/s]");
				relative_permittivity = param.get_double("Relative permittivity");
		  }
		  param.leave_subsection();
		}
		param.leave_subsection();
	}
	param.leave_subsection();
}


double
NAME::LiquidWater::get_DO2(){
	return oxygen_diffusion_coeff;
}

double
NAME::LiquidWater::get_DH(){
	return proton_diffusion_coeff;
}
double
NAME::LiquidWater::get_Relative_Permittivity(){
	return relative_permittivity;
}
