//---------------------------------------------------------------------------
//    $Id: newton_base.cc 1354 2013-08-17 00:01:22Z secanell $
//    Version: $Name:  $
//
//    Copyright (C) 2007 by Jason Boisvert
//
//    This file is subject to QPL and may not be  distributed
//    without copyright and license information. Please refer
//    to the file deal.II/doc/license.html for the  text  and
//    further information on this license.
//
//---------------------------------------------------------------------------

#include <newton_base.h>
#include <base/parameter_handler.h>
#include <base/data_out_base.h>
#include <lac/block_vector.h>

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>
#include <algorithm>
#include <optimization_block_matrix_application.h>

using namespace AppFrame;

newtonBase::newtonBase(ApplicationBase& app)
: ApplicationCopy(app),
default_residual(app),
compute_residual(&default_residual),
assemble_now(false),
assemble_threshold(0.),
debug_solution(false),
debug_update(false),
debug_residual(false),
debug(0)
{
	//deallog << "->Newton";
	//std::cout <<"In Newton Base!\n";
	
}

void
newtonBase::assemble()
{
    assemble_now = true;
}

double
newtonBase::threshold (double value)
{
    const double t = assemble_threshold;
    assemble_threshold = value;
    return t;
}

void
newtonBase::declare_parameters(ParameterHandler& param)
{
    param.enter_subsection("Newton");
	ReductionControl::declare_parameters (param);
	param.declare_entry("Assemble threshold", "0.", Patterns::Double());
	param.declare_entry("Debug level", "0", Patterns::Integer());
	param.declare_entry("Debug solution", "false", Patterns::Bool());
	param.declare_entry("Debug update", "false", Patterns::Bool());
	param.declare_entry("Debug residual", "false", Patterns::Bool());
    param.leave_subsection();
    
    ApplicationCopy::declare_parameters(param);
}

void
newtonBase::_initialize (ParameterHandler& param)
{
    param.enter_subsection("Newton");
	control.parse_parameters (param);
	assemble_threshold = param.get_double("Assemble threshold");
	debug = param.get_integer("Debug level");
	debug_solution = param.get_bool("Debug solution");
	debug_update   = param.get_bool("Debug update");
	debug_residual = param.get_bool("Debug residual");
    param.leave_subsection();
}

void
newtonBase::initialize (ParameterHandler& param)
{
    ApplicationCopy::initialize(param);
    _initialize(param);
	
}


double
newtonBase::residual (AppFrame::FEVector& dst, const AppFrame::FEVectors& src)
{
	//std::cout <<"Compute Residual!!\n";
	const double result = app->residual(dst, src);
	return result;
}


void newtonBase::solve(AppFrame::FEVector& u,
		   const AppFrame::FEVectors& in_vectors)
{
}

void
newtonBase::initialize_initial_guess(BlockVector<double>& dst)
{

}




