#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
mkPolcurve.py
Displays polarisation curve created by fcst in the parametric mode

Input:
    - Datafile, e.g. "input.dat"
      Format:
      %eval_id         V_cell         obj_fn 
       1            0.3   -2.380636381 
       2           0.35   -2.223311025 
       3            0.4   -2.015362082 
       4           0.45   -1.749832606 
       5            0.5   -1.430078875 
       6           0.55   -1.078322382 
       7            0.6  -0.7347111835 
       8           0.65  -0.4420970865 
       9            0.7  -0.2278052956 
      10           0.75 -0.09479962371 
      11            0.8 -0.02921816945 
"""
#! Polarization Curve
#! ###############################################

# Basic imports:
import sys, os, shutil
import glob
import re
import commands

from StringIO import StringIO # Create stringlike file objects

# Import scientific packages:
from pylab import *
import scipy
import scipy.io
import scipy.interpolate
import scipy.optimize


print "="*50
print "= Plot Polarization Curves"
print "-"*50

print "-"*50
print "= - Parse Commandline "
# Import command line option parser:
from optparse import OptionParser
# Setup of the command line options:
usage = "usage: %prog [options] filename.dat"
parser = OptionParser(usage)
options, filename = parser.parse_args(sys.argv[1:])
print "-"*50

print "-"*50
print "= - Setup of the pylab images"
fig_width_pt        = 246.0         # Get this from LaTeX using \showthe\columnwidth
inches_per_pt   = 1.0/72.27                 # Convert pt to inch
fig_width_cm        = 30.0
inches_per_cm   = 0.393
golden_mean = (sqrt(5)-1.0)/2.0             # Aesthetic ratio
fig_width = fig_width_cm*inches_per_cm      # width in inches
fig_height = fig_width*golden_mean          # height in inches
fig_size =  [fig_width,fig_height]
params = {
            'backend': 'ps',
#           'backend': 'svg',
            'axes.labelsize': 18,
            'text.fontsize': 18,
            'titlesize': 24,
            'legend.fontsize': 16,
            'xtick.labelsize': 16,
            'ytick.labelsize': 16,
            'text.usetex': True,
            'figure.figsize': fig_size
        }
rcParams.update(params)
print "-"*50


print "= -Load data"
print "Load", filename[0]
myPolData = scipy.loadtxt(filename[0],skiprows=1);
myPolData[:,1]=-myPolData[:,1]
print "-"*50

print "-"*50
print "Plot Pol Curve Data"
fig=figure()
host=fig.add_subplot(111)
par1=host.twinx()

p1, = host.plot(-myPolData[:,2],myPolData[:,1],'bo-',label='Polcurve')
p2, = par1.plot(-myPolData[:,2],-myPolData[:,2]*myPolData[:,1],'go--',label='Power Density')

host.set_xlabel("Current Density [$A/cm^2$]")
host.set_ylabel("Cell Voltage [$V$]")
title("Polarization Curve")
par1.set_ylabel("Power Density [$W/cm^2$]")

host.yaxis.label.set_color(p1.get_color())
host.set_ylim(0, 1)
host.tick_params(axis='y',colors=p1.get_color())
par1.yaxis.label.set_color(p2.get_color())
par1.tick_params(axis='y',colors=p2.get_color())


lines = [p1, p2]
host.legend(lines, [l.get_label() for l in lines],loc="upper left")
#legend(loc="upper left")



figure()
loglog(-myPolData[:,2],myPolData[:,1],'o-')
xlabel("Current Density [$A/cm^2$]")
ylabel("Cell Voltage [$V$]")
title("Polarization Curve")
show()
print "-"*50

print "-"*50
print "Plot Pol Curve Data"

figure()
plot(-myPolData[:,2],-myPolData[:,3],'og-',label='Cathode')
plot(-myPolData[:,2], myPolData[:,4],'xb--',label='Anode')
fill_between(-myPolData[:,2],-myPolData[:,3],-myPolData[:,2]*0,where=-myPolData[:,3]<=0,facecolor='blue' , interpolate=True)
fill_between(-myPolData[:,2],-myPolData[:,3],-myPolData[:,2]*0,where=-myPolData[:,3]>=0,facecolor='green', interpolate=True)
xlabel("Current Density [$A/cm^2$]")
ylabel("Water Crossover [$xxx$]")
title("Water Crossover")
legend(loc="upper right")
show()